from .calculateMetrics import *
import seaborn as sns
from .loadfile import *
import matplotlib.pyplot as plt
from statsmodels.sandbox.stats.multicomp import multipletests

def readIF(parameter,chr,normIF=True,custom_name="InteractionFrequency"):
    all = pd.read_csv(parameter,sep="\t",header=None)
    score = all[all[0] == chr]
    if normIF:
        beforlog = score[3].copy()
        afterlog = np.log1p(beforlog)
        score[3] = afterlog / np.mean(afterlog[afterlog>0])
    score.index = range(score.shape[0])
    score.columns = ["chr","start","end",custom_name]
    return(score)

def getMultiSamplesScore(sampleList, labels, res, chr, mode, UniqueParameter=None,smoothPC=True,logPC=False,
                        datatype="matrix",gt=None):
    if datatype == "rawhic" and not gt: raise ValueError("rawhic requires Genometable")
    if datatype == "rawhic" and mode != "IF":
        newlist = []
        for rawpath in sampleList:
            newpath = hic2matrix(rawpath,res,chr,gt)
            newlist.append(newpath)
        sampleList = newlist

    if mode == 'IS':
        if not UniqueParameter: UniqueParameter=300000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = InsulationScore(path,res,chr,square_size=UniqueParameter).getIS()
            else:
                next = InsulationScore(path,res,chr,square_size=UniqueParameter).getIS().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == 'raw':
        for i,path in enumerate(sampleList):
            if i==0: metricMT = pd.DataFrame(loadDenseMatrix(path).values.flatten())
            else:
                next = pd.DataFrame(loadDenseMatrix(path).values.flatten())
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "DI":
        if not UniqueParameter: UniqueParameter=1000000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = DirectionalityIndex(path,res,chr,DI_distance=UniqueParameter).getDI()
            else:
                next = DirectionalityIndex(path,res,chr,DI_distance=UniqueParameter).getDI().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "CI":
        if not UniqueParameter: UniqueParameter=300000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = ContrastIndex(path,res,chr,CI_size=UniqueParameter).getCI()
            else:
                next = ContrastIndex(path,res,chr,CI_size=UniqueParameter).getCI().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "SS":
        if not UniqueParameter: UniqueParameter=300000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = SeparationScore(path,res,chr,TADss_size=UniqueParameter).getTADss()
            else:
                next = SeparationScore(path,res,chr,TADss_size=UniqueParameter).getTADss().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "DLR":
        if not UniqueParameter: UniqueParameter=3000000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = DistalToLocalRatio(path,res,chr,sizeDLR=UniqueParameter).getDLR()
            else:
                next = DistalToLocalRatio(path,res,chr,sizeDLR=UniqueParameter).getDLR().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "PC1":
        if not UniqueParameter:
            warnings.warn("The sign of eigenvector is arbitrary unless specify a geneDensity file")
        for i,path in enumerate(sampleList):
            if i==0: metricMT = CompartmentPC1(path,res,chr).getPC1(signCorr = UniqueParameter)
            else:
                next = CompartmentPC1(path,res,chr).getPC1(signCorr = UniqueParameter,smooth = smoothPC, logOE=logPC).iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "IAS":
        if not UniqueParameter: UniqueParameter=300000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = intraTADscore(path,res,chr).getIntraS(IS_size = UniqueParameter)
            else:
                next = intraTADscore(path,res,chr).getIntraS(IS_size = UniqueParameter).iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "IAS":
        if not UniqueParameter: UniqueParameter=300000
        for i,path in enumerate(sampleList):
            if i==0: metricMT = interTADscore(path,res,chr).getInterS(IS_size = UniqueParameter)
            else:
                next = interTADscore(path,res,chr).getInterS(IS_size = UniqueParameter).iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    elif mode == "IF":
        for i,path in enumerate(sampleList):
            if i==0: metricMT = InteractionFrequency(path,res,chr,gt=gt,datatype=datatype).getIF()
            else:
                next = InteractionFrequency(path,res,chr,gt=gt,datatype=datatype).getIF().iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

    if mode != "raw":
        #metricMT.index = metricMT.start.tolist()
        #metricMT = metricMT.iloc[:,3:]
        metricMT.columns = ["chr","start","end"]+labels
    elif mode == "raw":
        metricMT.columns = labels
    return metricMT

class repQC:
    def __init__(self,pathlist,namelist,res,chr,mode,UniqueParameter,method="pearson",smoothPC=True,logPC=False,gt=None,datatype="matrix"):
        self.pathlist = pathlist
        self.namelist = namelist
        self.res = res
        self.chr = chr
        self.mode = mode
        self.UniqueParameter = UniqueParameter
        self.nScore = len(namelist)

        self.score = getMultiSamplesScore(self.pathlist,namelist,res=res,chr=chr,mode=mode,
                                        UniqueParameter=UniqueParameter,smoothPC=smoothPC,logPC=logPC,
                                        gt=gt,datatype=datatype)
        self.corrMT = self.score.iloc[:,3:].corr(method=method)

    def corr_plot(self):
        if self.mode!="raw":
            sns.clustermap(self.corrMT ,cmap="RdPu")

    def calcuRepScore(self):
        maxCorr = abs(np.nan_to_num(self.corrMT[self.corrMT<1])).max()
        minCorr = abs(np.array(self.corrMT)).min()
        return(maxCorr-minCorr)

    def heatmap(self,start,end,figs=(10,5),vmin=None,vmax=None):
        sbin = start//self.res
        ebin = end//self.res
        plt.figure(figsize=figs)
        plt.imshow(self.score.iloc[sbin:ebin,:].T,aspect="auto",interpolation='nearest',vmin=vmin,vmax=vmax)
        plt.colorbar()

    def heatmap_tri(self,hic_path,start,end,clmax=None,heatmin=None,plottype="heat"):
        sbin = start//self.res
        ebin = end//self.res
        if ebin ==0: ebin = self.score.shape[0]

        from .callDirectionalTAD import PlotTAD
        plt.figure(figsize=(10,9+self.nScore*1))
        plt.subplot2grid((5+self.nScore,11),(0,0),rowspan=5,colspan=10)
        hp = PlotTAD(hic_path,self.res,self.chr,start,end,clmax=clmax)
        hp.draw()

        df = self.score.iloc[sbin:ebin,3:].T
        if plottype == "heat":
            plt.subplot2grid((5+self.nScore,11),(5,0),rowspan=self.nScore//2,colspan=11)
            plt.imshow(df,aspect="auto",interpolation='none',cmap="Purples",vmin=heatmin)
            plt.yticks(range(len(self.namelist)),self.namelist)
        elif plottype == "line":
            plt.subplot2grid((5+self.nScore,11),(5,0),rowspan=self.nScore//4+1,colspan=11)
            df.columns = range(df.shape[1])
            print(df.T)
            df.T.plot(ax=plt.gca())
            plt.xlim(0,df.shape[1]-1)
            plt.legend()
        plt.xticks(np.arange(0,df.shape[1]+1,(df.shape[1])/5),hp.mark)

    def anova_like(self,start,end):
        sbin = start//self.res
        ebin = end//self.res
        df = self.score.iloc[sbin:ebin,3:].T

        nLoci = df.shape[1]
        nScore = df.shape[0]
        arrays = np.zeros(nLoci)*np.NaN
        for i in range(nLoci):
            if (i-2)<0 or (i+2)>=nLoci: continue
            df_i = df.iloc[:,i-2:i+3]
            df_ilist = [df_i.iloc[j] for j in range(nScore)]
            pvalue = stats.f_oneway(*df_ilist)[1]
            arrays[i] = pvalue

        qvalue = np.zeros(nLoci)*np.NaN
        qvalue[~np.isnan(arrays)] = multipletests(arrays[~np.isnan(arrays)], method='fdr_bh')[1]
        return(arrays,qvalue)

        #from statsmodels.sandbox.stats.multicomp import multipletests
        #qvalue=multipletests(arrays, method='fdr_bh')



import matplotlib.pyplot as plt
from .loadfile import *
import numpy as np
from matplotlib.colors import LinearSegmentedColormap
#from scipy import ndimage
from .calculateMetrics import *
from scipy.interpolate import make_interp_spline
import os


class PlotCommon(object):
    def __init__(self,path,resolution,chr="",startSite=0,endSite=0,clmin=0,clmax=None, \
                title="",other_parameter=0,ndsmooth=None,datatype="matrix",gt=""):
        if datatype == "rawhic":
            codepath = os.path.dirname(os.path.realpath(__file__))
            makeIntra = codepath+"/extract/makeMatrixIntra.sh"
            juicer = codepath+"/jc/jctool_1.11.04.jar"
            foldername = "./MatrixTemp"+str(random.random())
            os.system("bash "+makeIntra+" "+"KR"+" "+"."+" "+path+" "+str(resolution)+" "+gt+" "+juicer+" "+chr+" "+foldername+ ">> info.txt")
            path = foldername+"/"+str(resolution)+"/observed.KR."+chr+".matrix.gz"
        matrix = loadDenseMatrix(path).values

        self.path = path
        if ndsmooth:
            self.matrix = ndimage.median_filter(matrix,ndsmooth)
        else: self.matrix = matrix
        self.resolution = resolution
        self.startSite = startSite
        self.endSite = endSite
        #self.clim = (clmin,clmax)
        self.title = title
        sbin = int(startSite/resolution)
        ebin = int(endSite/resolution)
        self.chr = chr
        self.other_parameter = other_parameter
        self.datatype = "matrix"

        if endSite == 0:
            self.matrixRegion = np.nan_to_num(self.matrix.copy())
            ebin = self.matrix.shape[0]
        elif (ebin - sbin) < 5:
            print("The region you choosed is too small")
            exit(1)
        else:
            self.matrixRegion = np.nan_to_num(self.matrix[sbin:ebin+1,sbin:ebin+1])

        if clmax:
            self.clim = (clmin,clmax)
        else:
            self.clim = (clmin,np.median(self.matrixRegion)*10)

        self.sbin = sbin
        self.ebin = ebin

        position = self.resolution * np.arange(self.sbin,self.ebin+1,int((self.ebin-self.sbin)/5))
        self.mark = [str(x/1000000)+"M" for x in position]
        self.ticks_pos = np.arange(self.sbin,self.ebin+1,(self.ebin-self.sbin)/5)

cmap= LinearSegmentedColormap.from_list("custom1",['#FFFFFF', '#d10a3f'])

class PlotSquare(PlotCommon):
    def draw(self):
        fig = plt.figure(figsize=(5,5))
        plt.imshow(self.matrixRegion,clim= self.clim,cmap=cmap,interpolation="nearest", aspect=1)
        plt.title(self.title,fontsize=20)

        ticks_pos = list(range(0,self.ebin-self.sbin+1,int((self.ebin-self.sbin)/5)))

        plt.yticks(ticks_pos,self.mark)
        plt.xticks([])
        position=fig.add_axes([0.9, 0.73, 0.015, 0.15])
        plt.colorbar(fraction = 0.02,aspect=10,pad=-0.12,cax=position)

class PlotTri(PlotCommon):
    def draw(self,interpolation="nearest"):
        tri_matrix = ndimage.rotate(self.matrixRegion,45)
        plt.plot(dpi=300)
        plt.imshow(tri_matrix,clim= self.clim,cmap=cmap,interpolation=interpolation, aspect=1)
        plt.title(self.title,fontsize=20)
        tri_shape = tri_matrix.shape[0]
        plt.ylim(tri_shape/2,0-tri_shape/10)
        plt.xlim(0,tri_shape)

        plt.colorbar(fraction = 0.02,aspect=10,pad=-0.12)
        plt.plot([0,tri_shape/2,tri_shape],[tri_shape/2,0,tri_shape/2],"k-",linewidth=0.2)

        ticks_pos = np.arange(0,tri_shape+1,tri_shape/5)

        plt.xticks(ticks_pos,self.mark)
        plt.yticks([])

class PlotBedGraph(PlotTri):

    def draw(self,type,UniqueParameter=None,smoothPC=True,logPC=False,customfile="",scorelim=None,scorecolor=None):
        if type == 'IS':
            score = InsulationScore(self.path,self.resolution,self.chr,datatype=self.datatype).getIS().InsulationScore
            title = "InsulationScore"
        elif type == 'DI':
            score = DirectionalityIndex(self.path,self.resolution,self.chr,datatype=self.datatype).getDI().DirectionalityIndex
            title = "DirectionalityIndex"
        elif type == "CI":
            score = ContrastIndex(self.path,self.resolution,self.chr,datatype=self.datatype).getCI().ContrastIndex
            title = "ContrastIndex"
        elif type == "SS":
            score = SeparationScore(self.path,self.resolution,self.chr,datatype=self.datatype).getTADss().SeparationScore
            title = "SeparationScore"
        elif type == "DLR":
            score = DistalToLocalRatio(self.path,self.resolution,self.chr,datatype=self.datatype).getDLR().DistalToLocalRatio
            title = "DistalToLocalRatio"
        elif type == "PC1":
            score = CompartmentPC1(self.path,self.resolution,self.chr,datatype=self.datatype).getPC1(signCorr = UniqueParameter,smooth = smoothPC, logOE=logPC).CompartmentPC1
            title = "CompartmentPC1"
        elif type == "IAS":
            score = intraTADscore(self.path,self.resolution,self.chr,datatype=self.datatype).getIntraS().iloc[:,3]
            title = "intraTADscore"
        elif type == "IES":
            score = interTADscore(self.path,self.resolution,self.chr,datatype=self.datatype).getInterS(useOE=UniqueParameter).interTADscore
            title = "inerTADscore"
        elif type in ["custom","IF"]:
            score = pd.read_csv(UniqueParameter,sep="\t",header=None).iloc[:,3]
            title = "InteractionFrequency"

        scoreRegion = score[self.sbin:self.ebin+1]

        plt.figure(figsize=(10,10))
        #pad + fraction = -0.1, So the bedGraph figure width should be (1+0.1)*width of HiC
        plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
        super().draw()
        plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
        plt.title(title,fontsize=20)
        if scorecolor:
            plt.plot(scoreRegion,c=scorecolor)
        else:
            plt.plot(scoreRegion,c="dodgerblue")

        plt.xlim(self.sbin,self.ebin)
        if scorelim: plt.ylim(scorelim[0],scorelim[1])
        plt.plot([self.sbin,self.ebin],[score.median(),score.median()],"k--",linewidth=0.4)

        ticks_pos = np.arange(self.sbin,self.ebin+1,(self.ebin-self.sbin)/5)
        plt.xticks(ticks_pos,self.mark)

    def onlyMetric(self,mode,parameter,title,scorelim=None,scorecolor=None,tick=True,fill=False,linesmooth=None):
        from MultiTypeScore import multiScore
        score = multiScore(self.path,self.resolution,self.chr).obtainOneScore(mode=mode,parameter=parameter).iloc[:,3]
        if linesmooth:
            y = list(score[self.sbin:self.ebin+1])
            x = list(score[self.sbin:self.ebin+1].index)
            x_smooth = np.linspace(min(x),max(x),int(len(y)//linesmooth))
            y_smooth = make_interp_spline(x, y)(x_smooth)
            scoreRegion = pd.Series(y_smooth)
            scoreRegion.index = x_smooth
        else: scoreRegion = score[self.sbin:self.ebin+1]
        plt.title(title,fontsize=20)
        if scorecolor:
            plt.plot(scoreRegion,c=scorecolor)
        else:
            plt.plot(scoreRegion,c="dodgerblue")

        plt.xlim(self.sbin,self.ebin)
        if scorelim: plt.ylim(scorelim[0],scorelim[1])
        #plt.plot([self.sbin,self.ebin],[score.median(),score.median()],"k--",linewidth=0.4)
        if tick:
            plt.xticks(self.ticks_pos,self.mark)
        else:
            plt.xticks(self.ticks_pos,[])

        if fill:
            plt.plot([self.sbin,self.ebin],[0,0],"k--",linewidth=0.6)
            #np.arange(self.sbin,self.ebin+1,1)
            plt.fill_between(scoreRegion.index,scoreRegion, 0,\
                            where = scoreRegion <=0,facecolor='grey', alpha=0.5)
            plt.fill_between(scoreRegion.index,scoreRegion, 0,\
                            where = scoreRegion >=0,facecolor=scorecolor, alpha=0.4)

    def makePDF(self,type,PDFname):
        self.draw(type)
        plt.savefig(PDFname+".pdf")



from .loadfile import *
import argparse
from scipy import ndimage
from scipy import stats
from sklearn.decomposition import PCA
from sklearn import preprocessing
#from callDirectionalTAD import TADcallIS
import warnings
import os

class BasePara:
    def __init__(self,path,resolution,chromosome,out_name="noName",useNA=True,datatype="matrix"):
        if datatype == "rawhic":
            codepath = os.path.dirname(os.path.realpath(__file__))
            makeIntra = codepath+"/extract/makeMatrixIntra.sh"
            juicer = codepath+"/jc/jctool_1.11.04.jar"
            os.system("sh "+makeIntra+" "+"KR"+" "+"."+" "+path+" "+str(resolution)+" "+gt+" "+juicer+" "+chr)
            path = "./MatrixTemp/"+str(resolution)+"/observed.KR."+chromosome+".matrix.gz"
        self.path = path
        #self.matrixNA = loadDenseMatrix(path).values
        self.matrix = loadDenseMatrix(path,log=False).values
        self.matrix_shape = self.matrix.shape[0]
        self.resolution = resolution
        self.chromosome = chromosome
        self.out_name = out_name
        self.useNA = useNA
        self.allsum = np.nansum(self.matrix)
        if useNA == True:
            self.blankarray = np.zeros(self.matrix_shape) * np.NaN
        elif useNA == False:
            self.blankarray = np.zeros(self.matrix_shape)

    def makeDF(self,array,metrics_name="unknown metrics"):
        array = np.round(array,6)
        df = pd.DataFrame({metrics_name:array})
        df["chr"] = self.chromosome
        df["start"] = df.index * self.resolution
        df["end"] = df["start"] + self.resolution
        df = df.loc[:,["chr","start","end",metrics_name]]
        return df

    def makeCSV(self,df):
        print("--------Export to csv file--------- \n")
        df.to_csv(self.out_name + ".bedGraph", sep="\t", header=False, index=False)

class InteractionFrequency:
    def __init__(self,path,resolution,chromosome,gt=None,datatype="rawhic",normIF=True,out_name="noName"):
        if not gt:
            raise ValueError("Genometable is required for the calculation of IF")
        codepath = os.path.dirname(os.path.realpath(__file__))
        soft = codepath+"/InteractionFreq.sh"
        juicer = codepath+"/jc/jctool_1.11.04.jar"
        chrnum = chromosome.split("chr")[1]
        os.system("sh '"+soft+"' '"+juicer+"' "+path+" "+chrnum+" "+str(resolution)+" "+gt+" "+"IF_"+chromosome+"> info.txt") #in case of space
        score = pd.read_csv("IF_"+chromosome+".bedGraph",sep="\t",header=None)
        if normIF:
            beforlog = score[3].copy()
            afterlog = np.log1p(beforlog)
            score[3] = afterlog / np.mean(afterlog[afterlog>0])
        score.index = range(score.shape[0])
        score.columns = ["chr","start","end","InteractionFreq"]
        os.system("rm "+"IF_"+chromosome+".bedGraph")
        self.score = score

    def getIF(self):
        return(self.score)

class InsulationScore(BasePara):
    def __init__(self,path,resolution,chromosome,out_name="InsulationScore",useNA=True,square_size=300000,datatype="matrix"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.square_size = square_size
        #The default size in Homer IS is 150000

    #print the instanced class
    def __str__(self):
        return "InsulationScore(matrix_path = '" + str(self.path) +"',\n" + \
                "resolution = " + str(self.resolution) + ", chromosome = '" + \
                self.chromosome + "', " + "out_name = '" + self.out_name + \
                "' \n, square_size = " + str(self.square_size) + ")"
    __repr__ = __str__

    def getIS(self):
        squareBin = round(self.square_size/self.resolution)
        array = self.blankarray

        for i in range(self.matrix_shape):
            if(i - squareBin < 0 or i + squareBin >= self.matrix_shape): continue
            score = self.matrix[i-squareBin: i, i+1: i+squareBin+1].mean()
            if np.isnan(score) or score == 0: continue  #skip NaN value
            array[i] = score
        array = np.log1p(array/np.nanmean(array)) #normalization

        return super().makeDF(array,"InsulationScore")

    def getCSV(self):
        super().makeCSV(self.getIS())

def TADcallIS(matrixPath,resolution,chromosome,squareSize=300000,useNA=True):
    from scipy.signal import argrelextrema

    ISbedgraph = InsulationScore(matrixPath,resolution,chromosome,square_size=squareSize,useNA=useNA).getIS()
    ISoneNA = ISbedgraph.InsulationScore
    ISone = pd.Series(np.nan_to_num(ISoneNA))

    # local minimal
    localMinPos = argrelextrema(np.array(ISone), np.less)
    localMinIS = ISone.iloc[localMinPos]

    # 0< IS <0.8
    localMinIS = localMinIS[localMinIS!=0]
    localMinIS = localMinIS[localMinIS< np.mean(ISoneNA)]

    #Around TAD boundary
    binNum = int(100000/resolution)
    localMinAround = []
    aroundZero=[]
    diffrightleft=[]

    for i in localMinIS.index:
        # local minimal around 100kb
        localMinAround.append(ISone.loc[i-binNum:i+binNum].min())

        #IS_around !=0
        aroundZero.append(ISone.loc[i-binNum*2:i+binNum*2].min())

        #strong boundary
        minusbin = ISone.loc[i-binNum] - ISone.loc[i]
        plusbin = ISone.loc[i+binNum] - ISone.loc[i]
        diffrightleft.append(minusbin+plusbin)

    bool1 = (np.array(localMinIS)-np.array(localMinAround)) <= 0
    bool2 = np.array(aroundZero)>0
    bool3 = np.array(diffrightleft)>0.05
    localMinIS = localMinIS[bool1 * bool2 * bool3]

    # build a table as output
    TADnumber = len(localMinIS)
    chrlist = [chromosome] * (TADnumber-1)
    TADstart = (np.array(localMinIS.index)[:-1])*resolution
    TADend = (np.array(localMinIS.index)[1:])*resolution
    TADout = pd.DataFrame()
    TADout["chr"] = chrlist
    TADout["TADstart"] = TADstart
    TADout["TADend"] = TADend
    #Maximum TAD size 5MB, Minimum 0.3MB
    TADout = TADout[(TADout["TADend"]-TADout["TADstart"])<=5000000]
    TADout = TADout[(TADout["TADend"]-TADout["TADstart"])>=300000]

    withNA=[]
    for i in range(TADout.shape[0]):
        s = np.array(TADout.TADstart)[i] // resolution
        e = np.array(TADout.TADend)[i] // resolution
        whetherNAIS = np.isnan(sum(ISoneNA[s:e+1]))
        withNA.append(~whetherNAIS)
    TADout = TADout[withNA]

    return(TADout)

class ContrastIndex(BasePara):
    def __init__(self,path,resolution,chromosome,out_name="ContrastIndex",useNA=True,CI_size=200000,datatype="matrix"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.CI_size = CI_size

    #print the instanced class
    def __str__(self):
        return "ContrastIndex(matrix_path = '" + str(self.path) +"',\n" + \
                "resolution = " + str(self.resolution) + ", chromosome = '" + \
                self.chromosome + "', " + "out_name = '" + self.out_name + \
                "' \n, CI_size = " + str(self.CI_size) + ")"
    __repr__ = __str__

    def getCI(self):
        CI_Bin = round(self.CI_size/self.resolution)
        array = self.blankarray

        for i in range(self.matrix_shape):
            if(i - CI_Bin < 0 or i + CI_Bin >= self.matrix_shape): continue
            matA = self.matrix[i-CI_Bin:i,i-CI_Bin:i]
            matB = self.matrix[i+1:i+CI_Bin+1,i+1:i+CI_Bin+1]
            A = np.sum(np.triu(matA,1))
            B = np.sum(np.triu(matB,1))
            C = np.sum(self.matrix[i-CI_Bin: i, i+1: i+CI_Bin+1])
            if np.isnan(np.sum(A+B+C)) or min(A,B,C) == 0: continue  #skip NaN value
            array[i] = np.log1p(A+B) - np.log1p(C)

        return super().makeDF(array,"ContrastIndex")

    def getCSV(self):
        super().makeCSV(self.getCI())

class SeparationScore(BasePara):
    def __init__(self,path,resolution,chromosome,out_name="SeparationScore",useNA=True,TADss_size=300000,datatype="matrix"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.TADss_size = TADss_size

    def __str__(self):
        return "SeparationScore(matrix_path = '" + str(self.path) +"',\n" + \
                "resolution = " + str(self.resolution) + ", chromosome = '" + \
                self.chromosome + "', " + "out_name = '" + self.out_name + \
                "' \n, TADss_size = " + str(self.TADss_size) + ")"
    __repr__ = __str__

    def getTADss(self):
        TADss_Bin = round(self.TADss_size/self.resolution)
        array = self.blankarray

        for i in range(self.matrix_shape):
            if(i - TADss_Bin < 0 or i + TADss_Bin >= self.matrix_shape): continue
            matA = self.matrix[i-TADss_Bin:i,i-TADss_Bin:i]
            matB = self.matrix[i+1:i+TADss_Bin+1,i+1:i+TADss_Bin+1]
            A = np.triu(matA,1).sum()
            B = np.triu(matB,1).sum()
            C = self.matrix[i-TADss_Bin: i, i+1: i+TADss_Bin+1].sum()
            if np.isnan(np.sum(A+B+C)) or (min(A,B,C)==0): continue
            array[i] = C/(min(C+A,C+B))

        return super().makeDF(array,"SeparationScore")

    def getCSV(self):
        super().makeCSV(self.getTADss())

class DirectionalityIndex(BasePara):
    def __init__(self,path,resolution,chromosome,out_name="noName",useNA=True,DI_distance=1000000,datatype="matrix"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.DI_distance = DI_distance
        #The default distance in Homer DI is 1000000

    #print the instanced class
    def __str__(self):
        return "DirectionalityIndex(matrix_path = '" + str(self.path) +"',\n" + \
                "resolution = " + str(self.resolution) + ", chromosome = '" + \
                self.chromosome + "', " + "out_name = '" + self.out_name + \
                "' \n, DI_distance = " + str(self.DI_distance) + ")"
    __repr__ = __str__

    def getDI(self):
        distanceBin = round(self.DI_distance/self.resolution)
        array = self.blankarray

        for i in range(self.matrix_shape):
            if(i - distanceBin < 0 or i + distanceBin >= self.matrix_shape): continue
            A = self.matrix[i,i-distanceBin:i].sum()
            B = self.matrix[i,i+1:i+distanceBin+1].sum()
            E = (A+B)/2

            if np.isnan(np.sum(A+B)) or (B-A == 0) or (min(A,B)==0): continue #skip NaN value
            sign = (B-A)/abs(B-A)
            array[i] = sign * np.log1p(((A-E)**2)/E + ((B-E)**2)/E) #*****?

        return super().makeDF(array,"DirectionalityIndex")

    def getCSV(self):
        super().makeCSV(self.getDI())

class DistalToLocalRatio(BasePara):
    def __init__(self,path,resolution,chromosome,out_name="noName",useNA=True,sizeDLR=3000000,datatype="matrix"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.sizeDLR = sizeDLR

    def getDLR(self):
        sizeBin = round(self.sizeDLR/self.resolution)
        array = self.blankarray

        for i in range(self.matrix_shape):
            if(i - sizeBin < 0 or i + sizeBin >= self.matrix_shape): continue
            A = self.matrix[i,i-sizeBin:i].sum()
            B = self.matrix[i,i+1:i+sizeBin+1].sum()
            Aout = np.nansum(self.matrix[i,0:i-sizeBin])
            Bout = np.nansum(self.matrix[i,i+sizeBin+1:])

            if np.isnan(A+B) or min(A,B,Aout,Bout)==0: continue #skip NaN value
            array[i] = np.log(Aout+Bout)-np.log(A+B)
        return super().makeDF(array,"DistalToLocalRatio")

    def getCSV(self):
        super.makeCSV(self.getDLR())

'''
class interTADscore(BasePara):
    def getInterS(self,IS_size=300000,useNA=True,TADpath=None):
        if TADpath:
            usedPath = TADpath
        else:usedPath = self.path
        tad = TADcallIS(usedPath,self.resolution,self.chromosome,squareSize=IS_size,useNA=useNA)
        leftBorder =  np.array(tad.TADstart) // self.resolution
        rightBorder = np.array(tad.TADend) // self.resolution
        array = self.blankarray

        for i in range(self.matrix_shape):
            belongTAD = (i >= leftBorder) * (i < rightBorder)
            if sum(belongTAD) == 0: continue

            elif np.median(np.nan_to_num(self.matrix[i,:])) == 0:
                continue

            startBin = int(leftBorder[belongTAD])
            endBin = int(rightBorder[belongTAD])
            A = np.nansum(self.matrix[i,0:startBin-1])
            B = np.nansum(self.matrix[i,endBin+1:])
            if np.isnan(A+B) or max(A,B) == 0: continue
            array[i] = A+B

        #array = np.log1p(array/np.nanmean(array))
        array = (array/self.allsum)*1e4
        return super().makeDF(array,"interTADscore")
'''

class CompartmentPC1(BasePara):
    #def __init__(self,path,resolution,chromosome,out_name="noName"):
    #    super().__init__(path,resolution,chromosome,out_name)
    #    self.matrix = loadWithNorm(path,log = True).values

    def makeExpect(self,df,smooth):
        num = df.shape[0]
        avg=[]
        for i in range(num):  #间隔为i，格数为num-i
            dig=[]
            for j in range(num):
                if (j+i) < num: dig.append(df[j,j+i])
            avg.append(np.mean(dig))

        if smooth == True:
            for i in range(num):
                if avg[i] < df.mean(): #25000res 为10， 50000应该是10*（50000/25000)^2
                    for flank in range(num):
                        biggerBin = avg[i-flank:i+flank+1]
                        if np.sum(biggerBin)>=df.mean():
                            avg[i] = np.mean(biggerBin)
                            break

        expected = np.zeros((num, num))
        for i in range(num):
            for j in range(num):
                if np.isnan(df[i,j]):
                    expected[i,j] = np.NaN
                else:
                    distance = abs(i-j)
                    expected[i,j] = avg[distance]

        return(expected)

    def getPC1(self, logOE = False, signCorr = None, smooth = True):
        rawMT = np.nan_to_num(self.matrix)
        expectMT = self.makeExpect(rawMT,smooth)
        warnings.filterwarnings("ignore")
        oeMT = np.nan_to_num(rawMT / expectMT)
        warnings.filterwarnings("default")

        if logOE == True:
            oeMT = np.log(oeMT)
            oeMT[np.isinf(oeMT)] = 0

        allzero = oeMT.sum(axis=1) == 0
        notallzeroindex = np.array(range(self.matrix_shape))[~allzero]
        #pandas 比numpy慢很多
        #allzeroindex = np.array(range(oeMT.shape[0]))[allzero]
        #oeMT[allzeroindex,:] = np.NaN #把0值变成NA
        #oeMT[:,allzeroindex] = np.NaN
        #pearsonMT = np.array(pd.DataFrame(oeMT).corr())

        nonzeroMT = oeMT[~allzero,:][:,~allzero]
        nonzeroPearson = np.corrcoef(nonzeroMT)
        pca = PCA(n_components=5)
        trained = pca.fit(nonzeroPearson)
        pc1 = trained.components_
        noNAarray = pc1[0,:]

        array = np.zeros(self.matrix_shape)
        array[notallzeroindex] = noNAarray
        #pearsonMT = pd.DataFrame(np.zeros((self.matrix_shape,self.matrix_shape)) *np.NaN)
        #pearsonMT.iloc[notallzeroindex,notallzeroindex] = nonzeroPearson
        #pearsonMT = np.array(pearsonMT)

        #naPos = np.isnan(pearsonMT).all(axis=1)
        #pca = PCA(n_components=5)
        #trained = pca.fit(np.nan_to_num(pearsonMT))
        #pc1 = trained.components_
        #array = pc1[0,:]

        if signCorr == None:
            pass
        else:
            geneDensity = pd.read_csv(signCorr,header=None,sep='\t')
            gd = geneDensity[geneDensity[0] == self.chromosome][3]
            cor2gd = stats.spearmanr(gd,array)[0]
            if cor2gd <0: array = -array

        if self.useNA == True: array[allzero] = np.NaN
        return super().makeDF(array,"CompartmentPC1")

    def getCSV(self):
        super.makeCSV(self.getPC1())

class intraTADscore(CompartmentPC1):
    def getIntraS(self,IS_size=300000,useNA=True,TADpath=None,useOE=True,smooth=False,normTAD=True,TADfile=None):   #this useNA is for TAD calling
        if TADpath:
            usedPath = TADpath
        else:usedPath = self.path

        if TADfile:
            print('Use custom TAD region')
            alltad=pd.read_csv(TADfile,sep="\t",header=None,names=['chr','TADstart','TADend'])
            tad=alltad[alltad.chr==self.chromosome]
        else:
            print('Use bulit-in TAD calling')
            tad = TADcallIS(usedPath,self.resolution,self.chromosome,squareSize=IS_size,useNA=useNA)
        leftBorder =  np.array(tad.TADstart) // self.resolution
        rightBorder = np.array(tad.TADend) // self.resolution
        array = self.blankarray

        if useOE == True:
            rawMT = np.nan_to_num(self.matrix)
            expectMT = self.makeExpect(rawMT,smooth)
            warnings.filterwarnings("ignore")
            mt = np.nan_to_num(rawMT / expectMT)
            warnings.filterwarnings("default")
        else:
            mt = self.matrix

        for i in range(self.matrix_shape):
            belongTAD = (i >= leftBorder) * (i < rightBorder)
            if sum(belongTAD) == 0: continue
            #elif np.median(np.nan_to_num(self.matrix[i,:])) == 0:
            #    continue
            startBin = int(leftBorder[belongTAD])
            endBin = int(rightBorder[belongTAD])
            A = mt[i,startBin:i].sum()
            B = mt[i,i+1:endBin+1].sum()
            if np.isnan(A+B) or max(A,B) == 0: continue
            if normTAD == True:
                array[i] = (A+B)/(endBin-startBin)
            else: array[i] = A+B

        #array = np.log1p(array/np.nanmean(array))
        if useOE == False:
            array = (array/self.allsum)*1e4
        return super().makeDF(array,"IntraTADscore")

class interTADscore(CompartmentPC1):
    def getInterS(self,IS_size=300000,useNA=True,TADpath=None,useOE=True,
                logOE=False,smooth=False,normTAD=True,smoothScore=0.8,TADfile=None):   #this useNA is for TAD calling
        if TADpath:
            usedPath = TADpath
        else:usedPath = self.path

        if TADfile:
            alltad=pd.read_csv(TADfile,sep="\t",header=None,names=['chr','TADstart','TADend'])
            tad=alltad[alltad.chr==self.chromosome]
        else:
            tad = TADcallIS(usedPath,self.resolution,self.chromosome,squareSize=IS_size,useNA=useNA)
        leftBorder =  np.array(tad.TADstart) // self.resolution
        rightBorder = np.array(tad.TADend) // self.resolution
        array = self.blankarray

        if useOE == True:
            rawMT = np.nan_to_num(self.matrix)
            expectMT = self.makeExpect(rawMT,smooth)
            warnings.filterwarnings("ignore")
            mt = np.nan_to_num(rawMT / expectMT)
            warnings.filterwarnings("default")
        else:
            mt = self.matrix

        if logOE == True:
            mt = np.log(mt)
            mt[np.isinf(mt)] = 0

        for i in range(self.matrix_shape):
            belongTAD = (i >= leftBorder) * (i < rightBorder)
            if sum(belongTAD) == 0: continue
            #elif np.median(np.nan_to_num(self.matrix[i,:])) == 0:
            #    continue
            startBin = int(leftBorder[belongTAD])
            endBin = int(rightBorder[belongTAD])
            A = np.nansum(mt[i,0:startBin-1])
            B = np.nansum(mt[i,endBin+1:])
            if np.isnan(A+B) or max(A,B) == 0: continue
            if normTAD == True:
                array[i] = (A+B)/(self.matrix_shape-(endBin-startBin))
            else: array[i] = A+B
        if smoothScore:
            from scipy.ndimage import gaussian_filter1d
            array = gaussian_filter1d(array,smoothScore)

        #array = np.log1p(array/np.nanmean(array))
        if useOE == False:
            array = (array/self.allsum)*1e4
        return super().makeDF(array,"InterTADscore")



from .calculateMetrics import *
from scipy.signal import argrelextrema

class call_stripe(BasePara):
    def callStripe(self,squareSize=300000,useNA=True,seg=100000, strong_thresh= 0.2):
        iasDF = intraTADscore(self.path,self.resolution,self.chromosome).getIntraS()

        ias = iasDF.iloc[:,3]
        #local maximum
        localMaxIAS = ias.iloc[argrelextrema(np.array(ias), np.greater)]

        # IAS > mean
        localMaxIAS = localMaxIAS[localMaxIAS > ias.mean()]

        resolution = self.resolution
        binNum = int(seg/resolution)
        localMaxAround = []
        aroundZero=[]
        diffrightleft=[]

        for i in localMaxIAS.index:
            #maximum around 100kb
            localMaxAround.append(ias.loc[i-binNum:i+binNum].max())

            #around != NA ??

            #Strong peaks
            minusbin =  ias.loc[i] - ias.loc[i-binNum]
            plusbin = ias.loc[i] - ias.loc[i+binNum]
            diffrightleft.append(minusbin+plusbin)

        bool1 = (np.array(localMaxIAS)-np.array(localMaxAround)) >= 0
        bool2 = np.array(diffrightleft) > strong_thresh
        localMaxIAS = localMaxIAS[bool1*bool2]

        outDF = iasDF.loc[localMaxIAS.index]
        return outDF



import matplotlib.pyplot as plt
from .loadfile import *
import numpy as np
from matplotlib.colors import LinearSegmentedColormap
from scipy import ndimage
from .calculateMetrics import *
from .calculateTwoSample import *
import os
#from calcuDiffCI import *

cmap= LinearSegmentedColormap.from_list("custom2",['#1310cc', '#FFFFFF', '#d10a3f'])

class DiffDraw(object):
    def __init__(self,path,control_path,resolution,chr="",startSite=0,endSite=0,clmin=-2,clmax=2, \
                title="", startDRF=500000,sizeDRF=2000000,sizeIS=300000,sizeDCI=300000,datatype="matrix",gt=None):

        if datatype == "matrix":
            self.path = path
            self.control_path = control_path
        elif datatype == "rawhic":
            if not gt:
                raise ValueError("Genometable is required for the calculation of IF")
            self.path = hic2matrix(path,resolution,chr,gt)
            self.control_path = hic2matrix(control_path,resolution,chr,gt)
        else: print("Error, please use the matrix or rawhic datatype")
        print(self.path,"diff")
        treat = loadWithNorm(self.path,log = True).values
        control = loadWithNorm(self.control_path,log = True).values
        smooth = 2
        self.treat = ndimage.median_filter(treat,smooth)
        self.control = ndimage.median_filter(control,smooth)
        self.matrix = self.treat-self.control

        self.resolution = resolution
        self.startSite = startSite
        self.endSite = endSite
        self.clim = (clmin,clmax)
        self.title = title
        sbin = int(startSite/resolution)
        ebin = int(endSite/resolution)
        self.chr = chr
        self.sizeDCI = sizeDCI
        self.sizeDRF = sizeDRF
        self.startDRF = startDRF
        self.sizeIS = sizeIS

        if endSite == 0:
            self.plotall = True
            self.matrixRegion = self.matrix.copy()
            self.matrixRegion = np.nan_to_num(self.matrixRegion)
            ebin = self.matrix.shape[0]
        elif (ebin - sbin) < 5:
            print("The region you choosed is too small")
            exit(1)
        else:
            self.plotall = False
            self.matrixRegion = self.matrix[sbin:ebin+1,sbin:ebin+1]

        self.sbin = sbin
        self.ebin = ebin

        position = self.resolution * np.arange(self.sbin,self.ebin+1,int((self.ebin-self.sbin)/5))
        self.mark = [str(x/1000000)+"M" for x in position]

    def draw_square(self):
        fig = plt.figure(figsize=(5,5))
        plt.imshow(self.matrixRegion,clim= self.clim,cmap=cmap,interpolation="nearest", aspect=1)
        plt.title(self.title)

        ticks_pos = list(range(0,self.ebin-self.sbin+1,int((self.ebin-self.sbin)/5)))

        plt.yticks(ticks_pos,self.mark)
        plt.xticks([])
        position=fig.add_axes([0.9, 0.73, 0.015, 0.15])
        plt.colorbar(fraction = 0.02,aspect=10,pad=-0.12,cax=position)

    def draw_tri(self):
        tri_matrix = ndimage.rotate(self.matrixRegion,45)
        plt.plot(dpi=300)
        plt.imshow(tri_matrix,clim= self.clim,cmap=cmap,interpolation="nearest", aspect=1)
        plt.title(self.title,fontsize=20)
        tri_shape = tri_matrix.shape[0]
        plt.ylim(tri_shape/2,0-tri_shape/10)
        plt.xlim(0,tri_shape)

        plt.colorbar(fraction = 0.02,aspect=10,pad=-0.12)
        plt.plot([0,tri_shape/2,tri_shape],[tri_shape/2,0,tri_shape/2],"k-",linewidth=0.2)

        ticks_pos = np.arange(0,tri_shape+1,tri_shape/5)

        plt.xticks(ticks_pos,self.mark)
        plt.yticks([])

    def drawTAD(self,squareSize=300000,useNA=True):
        Tad = TADcallIS(self.control_path,self.resolution,self.chr,squareSize,useNA=useNA)
        selectTADbool = np.logical_and(Tad["TADstart"] >= self.startSite,Tad["TADend"] <= self.endSite)
        selectTAD=Tad[selectTADbool]

        originalWidth = self.matrixRegion.shape[0]
        figwidth = ndimage.rotate(self.matrixRegion,45).shape[0]

        xpos=[]
        ypos=[]
        for i in range(selectTAD.shape[0]):
            left = ((selectTAD["TADstart"].iloc[i]-self.startSite)/self.resolution+1) * (figwidth/originalWidth)
            right = ((selectTAD["TADend"].iloc[i]-self.startSite)/self.resolution+1) * (figwidth/originalWidth)
            xpos.append(left)
            xpos.append((left+right)/2)
            xpos.append(right)

            ypos.append(figwidth/2)
            ypos.append(figwidth/2-(right-left)/2)
            ypos.append(figwidth/2)

        self.draw_tri()
        plt.plot(xpos,ypos,"k-",linestyle ="dashed")

    def drawMetric(self,type,customfile="",name="custom"):
        if type == "deltaDLR":
            score = deltaDLR(self.path,self.control_path,self.resolution,self.chr, \
                    sizeDLR=3000000).getDeltaDLR().deltaDLR
            title = "deltaDLR"
        elif type == "ISC":
            score = TADScoreChange(self.path,self.control_path,self.resolution,
                                    self.chr).getChange("IS",300000).InsulationScoreChange
            title = "InsulationScoreChange"
        elif type == "CIC":
            score = TADScoreChange(self.path,self.control_path,self.resolution,
                                    self.chr).getChange("CI",300000).ContrastIndexChange
            title = "ContrastIndexChange"
        elif type == "DRF":
            score = DirectionalRelativeFreq(self.path,self.control_path,self.resolution,self.chr, \
                    start_distance=self.startDRF,end_distance=self.sizeDRF).getDRF().DirectionalRelativeFrequency
            title = "DirectionalRelativeFreq"
        elif type == "intraSC":
            score = intraScoreChange(self.path,self.control_path,self.resolution,self.chr).getIntraSC().IntraSC
            title = "intraScoreChange"
        elif type == "interSC":
            score = interScoreChange(self.path,self.control_path,self.resolution,self.chr).getInterSC().InterSC
            title = "interScoreChange"
        elif type == "custom":
            score = pd.read_csv(customfile,sep="\t",header=None).iloc[:,3]
            title = name

        scoreRegion = score[self.sbin:self.ebin+1]
        plt.figure(figsize=(10,10))
        #pad + fraction = -0.1, So the bedGraph figure width should be (1+0.1)*width of HiC
        plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
        self.draw_tri()
        plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
        plt.title(title,fontsize=20)
        plt.plot(scoreRegion,c='black')
        plt.xlim(self.sbin,self.ebin)
        plt.plot([self.sbin,self.ebin],[0,0],"k--",linewidth=0.4)
        if self.plotall: self.ebin = self.ebin-1
        plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion,\
                        where = scoreRegion >0,facecolor='#e9a3c9', alpha=0.5)

        plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion,\
                        where = scoreRegion <0,facecolor='#a1d76a', alpha=0.5)

        ticks_pos = np.arange(self.sbin,self.ebin+1,(self.ebin-self.sbin)/5)
        plt.xticks(ticks_pos,self.mark)
        os.system("rm -rf MatrixTemp0*")


    def draw_DRF(self):  #已被上面的替代
        score = DirectionalRelativeFreq(self.path,self.control_path,self.resolution,self.chr, \
                start_distance=self.startDRF,end_distance=self.sizeDRF).getDRF().DirectionalRelativeFreq
        title = "DirectionalRelativeFreq"
        scoreRegion = score[self.sbin:self.ebin+1]

        plt.figure(figsize=(10,10))
        #pad + fraction = -0.1, So the bedGraph figure width should be (1+0.1)*width of HiC
        plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
        self.draw_tri()
        plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
        plt.title(title,fontsize=20)
        plt.plot(scoreRegion,c='black')
        plt.xlim(self.sbin,self.ebin)
        plt.plot([self.sbin,self.ebin],[score.mean(),score.mean()],"k--",linewidth=0.4)
        if self.plotall: self.ebin = self.ebin-1
        plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion,\
                        where = scoreRegion >0,facecolor='#e9a3c9', alpha=0.99)

        plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion,\
                        where = scoreRegion <0,facecolor='#a1d76a', alpha=0.99)

        ticks_pos = np.arange(self.sbin,self.ebin+1,(self.ebin-self.sbin)/5)
        plt.xticks(ticks_pos,self.mark)

    def draw_diffCI(self):
        score = DiffCI(self.path,self.control_path,self.resolution,self.chr,diffCI_size=self.sizeDCI).getDiffCI().diffCI
        title = "Diff CI"
        scoreRegion = score[self.sbin:self.ebin+1]

        plt.figure(figsize=(10,10))
        #pad + fraction = -0.1, So the bedGraph figure width should be (1+0.1)*width of HiC
        plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
        self.draw_tri()
        ciplot= plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
        plt.title(title,fontsize=20)
        plt.plot(scoreRegion,c='black')
        plt.xlim(self.sbin,self.ebin)
        plt.plot([self.sbin,self.ebin],[0,0],"k--",linewidth=0.6)
        if np.mean(scoreRegion)<0:
            plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion, 0,\
                            where = scoreRegion <0,facecolor='blue', alpha=0.5)
        else:
            plt.fill_between(np.arange(self.sbin,self.ebin+1,1),scoreRegion, 0,\
                            where = scoreRegion >0,facecolor='red', alpha=0.5)


        ticks_pos = np.arange(self.sbin,self.ebin+1,(self.ebin-self.sbin)/5)
        plt.xticks(ticks_pos,self.mark)



from .calculateMetrics import *
from scipy.signal import argrelextrema
from .plotMetrics import *
from .plotTwoSample import *
import sys

class PlotTAD(PlotTri):
    def drawTAD(self,squareSize=300000,useNA=True):
        Tad = TADcallIS(self.path,self.resolution,self.chr,squareSize,useNA=useNA)
        selectTADbool = np.logical_and(Tad["TADstart"] >= self.startSite,Tad["TADend"] <= self.endSite)
        selectTAD=Tad[selectTADbool]

        originalWidth = self.matrixRegion.shape[0]
        figwidth = ndimage.rotate(self.matrixRegion,45).shape[0]

        xpos=[]
        ypos=[]
        for i in range(selectTAD.shape[0]):
            left = ((selectTAD["TADstart"].iloc[i]-self.startSite)/self.resolution+1) * (figwidth/originalWidth)
            right = ((selectTAD["TADend"].iloc[i]-self.startSite)/self.resolution+1) * (figwidth/originalWidth)
            xpos.append(left)
            xpos.append((left+right)/2)
            xpos.append(right)

            ypos.append(figwidth/2)
            ypos.append(figwidth/2-(right-left)/2)
            ypos.append(figwidth/2)

        super().draw()
        plt.plot(xpos,ypos,"k-",linestyle ="dashed")

    def makePDF(self,PDFname,squareSize=150000):
        self.drawTAD(squareSize)
        plt.savefig(PDFname+".pdf")

class stripeTAD(BasePara):
    def callStripe(self,squareSize=300000,useNA=True,seg=100000):
        Tad = TADcallIS(self.path,self.resolution,self.chromosome,squareSize,useNA=useNA)
        intraScore = intraTADscore(self.path,self.resolution,self.chromosome).getIntraS().iloc[:,3]
        nonNAIntraScore = intraScore[~intraScore.isnull()]
        bm = nonNAIntraScore.sample(300,random_state=1)

        pl = []
        pr = []
        import statsmodels.stats.weightstats as sw
        from statsmodels.sandbox.stats.multicomp import multipletests
        for i in range(Tad.shape[0]):
            regionLeft = int((Tad.iloc[i,1])/self.resolution)
            regionRight = int((Tad.iloc[i,2])/self.resolution)
            scorei = intraScore.iloc[regionLeft:regionRight]
            segbin = int(seg/self.resolution) #bins of the corner
            l = scorei.iloc[0:segbin]
            r = scorei.iloc[-segbin:]

            pvalue_l = sw.ztest(bm, value=l.mean(), alternative="smaller")[1]
            pvalue_r = sw.ztest(bm, value=r.mean(), alternative="smaller")[1]
            pl.append(pvalue_l)
            pr.append(pvalue_r)

        qvalue_l = multipletests(pl, method='bonferroni')[1]
        qvalue_r = multipletests(pr, method='bonferroni')[1]
        status = []
        for i in range(Tad.shape[0]):
            if qvalue_l[i] < 0.05 and qvalue_r[i] >0.05:
                status.append("leftStripe")
            elif qvalue_r[i] <0.05 and qvalue_l[i] > 0.05:
                status.append("rightStripe")
            elif qvalue_r[i] <0.05 and qvalue_l[i] < 0.05:
                status.append("loopTAD")
            else:
                status.append("otherTAD")
        Tad["TADtype"] = status
        return(Tad)
        #m = scorei.iloc[seg:-seg]
        #ls = l.min() > m.mean() and l.mean() > interScore.median()
        #rs = r.min() > m.mean() and r.mean() > interScore.median()

class DirectionalTAD(DiffDraw):
    def extractRegion(self):
        ControlTad = TADcallIS(self.control_path,self.resolution,self.chr,squareSize= self.sizeIS)
        drf = DirectionalRelativeFreq(self.path,self.control_path,self.resolution,self.chr, \
                start_distance=self.startDRF,end_distance=self.sizeDRF).getDRF().iloc[:,3]

        MinDRF=[]
        MaxDRF=[]
        for i in range(ControlTad.shape[0]):
            regionLeft = int((ControlTad.iloc[i,1])/self.resolution)
            regionRight = int((ControlTad.iloc[i,2])/self.resolution)
            MinDRF.append(drf.iloc[regionLeft:regionRight].min())
            MaxDRF.append(drf.iloc[regionLeft:regionRight].max())

        leftTAD = ControlTad[np.nan_to_num(MaxDRF)<0]
        rightTAD = ControlTad[np.nan_to_num(MinDRF)>0]
        return(leftTAD,rightTAD,drf)

    def plotAlldirec(self,type):
        if type == "left":
            region,_,drf = self.extractRegion()
        elif type == "right":
            _,region,drf = self.extractRegion()
        else:
            sys.exit("Please specific 'left' or 'right' mode")

        #region = region.iloc[0:2,:]
        num = region.shape[0]
        plt.figure(figsize=(7*num, 7+7*(10/6)))

        for i in range(num):
            TADleft = region.iloc[i,1] # directional TAD region
            TADright = region.iloc[i,2]
            start = TADleft - 1000000 # plot region
            end = TADright + 1000000
            sbin = int(start/self.resolution)
            ebin = int(end/self.resolution)
            drfRegion = drf[sbin:ebin+1]

            plotleft = ((TADleft-start)/self.resolution+1)*1.4142 #plot region of TAD
            plotright = ((TADright-start)/self.resolution+1)*1.4142

            #plot1: diffiential contact matrix
            ax1 = plt.subplot2grid((17,11*num),(0,i*11),rowspan=5,colspan=10)
            difftri = DiffDraw(self.path,self.control_path,self.resolution,startSite=start, \
                        endSite=end, clmin=-2,clmax=2,chr=self.chr,title="#"+str(i+1)+" Directional TAD")
            difftri.draw_tri()

            ## show the TAD
            TAD_xpos = [plotleft,(plotleft+plotright)/2,plotright]
            TAD_ypos = [ax1.get_ylim()[0],ax1.get_ylim()[0]-(plotright-plotleft)/2,ax1.get_ylim()[0]]
            ax1.plot(TAD_xpos,TAD_ypos,"k-")

            #plot2: DRF score
            ax2 = plt.subplot2grid((17,11*num),(5,i*11),rowspan=1,colspan=11)
            plt.title("DirectionalRelativeFreq",fontsize=20)
            plt.plot(drfRegion,c='black')
            plt.xlim(sbin,ebin)
            plt.plot([sbin,ebin],[drf.mean(),drf.mean()],"k--",linewidth=0.4)
            plt.fill_between(np.arange(sbin,ebin+1,1),drfRegion,\
                            where = drfRegion >0,facecolor='#e9a3c9', alpha=0.99)

            plt.fill_between(np.arange(sbin,ebin+1,1),drfRegion,\
                            where = drfRegion <0,facecolor='#a1d76a', alpha=0.99)

            ticks_pos = np.arange(sbin,ebin+1,(ebin-sbin)/5)
            plt.xticks(ticks_pos,difftri.mark)

            #plot3: Control matrix
            ax3 = plt.subplot2grid((17,11*num),(7,i*11),rowspan=5,colspan=10)
            #PlotTAD(self.control_path,self.resolution,startSite=start,endSite=end, \
            #    clmin=0,clmax=50,chr=self.chr,title="#"+str(i+1)+" Contact matrix of Control").drawTAD(self.sizeIS)
            PlotTri(self.control_path,self.resolution,startSite=start,endSite=end,
                clmin=0,clmax=50,chr=self.chr,title="#"+str(i+1)+" Contact matrix of Control").draw()
            ax3.plot(TAD_xpos,TAD_ypos,"k-")

            #plot4: Treated matrix
            ax4 = plt.subplot2grid((17,11*num),(12,i*11),rowspan=5,colspan=10)
            PlotTri(self.path,self.resolution,startSite=start,endSite=end,
                clmin=0,clmax=50,chr=self.chr,title="#"+str(i+1)+" Contact matrix of treated").draw()
            ax4.plot(TAD_xpos,TAD_ypos,"k-")

    def makePDF(self,type,outname):
        self.plotAlldirec(type)
        plt.savefig(outname,bbox_inches='tight')



from .calculateMetrics import *
from .calculateTwoSample import *
import matplotlib.pyplot as plt
import seaborn as sns
from .plotTwoSample import *
from scipy import stats
import os
import subprocess
#from callDirectionalTAD import *
#from plotMetrics import *
#from hmmlearn import hmm

class multiScore:
    def __init__(self,path,res,chr,control_path=""):
        self.rawpath = path # store for IF
        self.path = path
        self.res = res
        self.chr = chr
        self.rawcontrol = control_path
        self.control_path = control_path



    def obtainOneScore(self,mode,parameter=None,smoothPC=True,logPC=False,
                        custom_name="InteractionFrequency",normIF=True,gt=None,datatype="matrix",TADfile=None,
                        msi='fithic2'):
        if datatype == "rawhic" and not gt: raise ValueError("rawhic requires Genometable")

        if datatype == "rawhic" and mode != "IF":
            self.path = hic2matrix(self.path,self.res,self.chr,gt)
        elif datatype == "cool" and mode != "IF":
            self.path = cool2matrix(self.path,self.res,self.chr,gt)
            print(self.path)

        if mode == "IS":
            if not parameter: parameter=300000
            score = InsulationScore(self.path,self.res,self.chr,square_size=int(parameter)).getIS()
        elif mode == "CI":
            if not parameter: parameter=300000
            score = ContrastIndex(self.path,self.res,self.chr,CI_size=int(parameter)).getCI()
        elif mode == "DI":
            if not parameter: parameter=1000000
            score = DirectionalityIndex(self.path,self.res,self.chr,DI_distance=int(parameter)).getDI()
        elif mode == "SS":
            if not parameter: parameter=300000
            score = SeparationScore(self.path,self.res,self.chr,TADss_size=int(parameter)).getTADss()
        elif mode == "DLR":
            if not parameter: parameter=3000000
            score = DistalToLocalRatio(self.path,self.res,self.chr,sizeDLR=int(parameter)).getDLR()
        elif mode == "IAS":
            if not parameter: parameter=300000
            score = intraTADscore(self.path,self.res,self.chr).getIntraS(IS_size = int(parameter),TADfile=TADfile)
        elif mode == "IES":
            if not parameter: parameter=300000
            score = interTADscore(self.path,self.res,self.chr).getInterS(IS_size = int(parameter),TADfile=TADfile)
        elif mode == "PC1":
            if not parameter:
                warnings.warn("The sign of eigenvector is arbitrary unless specify a geneDensity file")
            score = CompartmentPC1(self.path,self.res,self.chr).getPC1(signCorr = parameter,smooth = smoothPC, logOE=logPC)
        elif mode == "IF":
            if not gt:
                raise ValueError("Genometable is required for the calculation of IF")
            codepath = os.path.dirname(os.path.realpath(__file__))
            juicer = codepath+"/jc/jctool_1.11.04.jar"
            chrnum = self.chr.split("chr")[1]

            if msi == 'hiccups':
                soft = codepath+"/extract/hiccup.sh"
                os.system("sh '"+soft+"' '"+juicer+"' "+self.path+" "+chrnum+" "+str(self.res)+" "+gt+" "+"IF_"+self.chr)
            else:
                soft = codepath+"/InteractionFreq.sh"
                os.system("sh '"+soft+"' '"+juicer+"' "+self.path+" "+chrnum+" "+str(self.res)+" "+gt+" "+"IF_"+self.chr) #in case of space
                
            score = pd.read_csv("IF_"+self.chr+".bedGraph",sep="\t",header=None)
            if normIF:
                beforlog = score[3].copy()
                afterlog = np.log1p(beforlog)
                score[3] = afterlog / np.mean(afterlog[afterlog>0])
            score.index = range(score.shape[0])
            score.columns = ["chr","start","end",custom_name]
            os.system("rm "+"IF_"+self.chr+".bedGraph")

        elif mode == "custom":
            all = pd.read_csv(parameter,sep="\t",header=None)
            score = all[all[0] == self.chr]
            if normIF:
                beforlog = score[3].copy()
                afterlog = np.log1p(beforlog)
                score[3] = afterlog / np.mean(afterlog[afterlog>0])
            score.index = range(score.shape[0])
            score.columns = ["chr","start","end",custom_name]
        elif mode == "stripe":
            score = stripeTAD(self.path,self.res,self.chr).callStripe(seg=parameter)
        else: print("Error: Please use the right mode"); exit(1)

        return(score)

    def allOneScore(self,typelist=["IS","CI","DI","TADss","DLR","intraS","interS","PC1","custom"],
                    parameterlist=[300000,300000,1000000,300000,3000000,300000,300000,"NotSpecified","customPath"],
                    smoothPC=True,logPC=False,normIF=True,datatype="matrix",gt=None):
        for i,type in enumerate(typelist):
            if i == 0:
                multiType = self.obtainOneScore(mode=typelist[i],parameter=parameterlist[i],smoothPC=smoothPC,logPC=logPC,
                                                datatype=datatype,gt=gt)
            else:
                if datatype=="rawhic" and self.path !=self.rawpath:
                    datatype2="matrix"
                else: datatype2 =datatype
                if typelist[i] == "IF":
                    self.path = self.rawpath; datatype2="rawhic"
                next = self.obtainOneScore(mode=typelist[i],parameter=parameterlist[i],smoothPC=smoothPC,logPC=logPC,
                                        datatype=datatype2,gt=gt).iloc[:,3]
                multiType = pd.concat([multiType,next],axis=1)

        return(multiType)

    def plotOneScore(self,typelist=["IS","CI","DI","TADss","DLR","intraS","interS","PC1","custom"],
                    parameterlist=[300000,300000,1000000,300000,3000000,300000,300000,"NotSpecified","customPath"],
                    start=0,end=0,clmax=100,plotTAD=False,smoothPC=True,logPC=False,datatype="matrix",gt=None):
        import matplotlib.colors as mcolors
        from .callDirectionalTAD import PlotTAD

        cols = list(mcolors.TABLEAU_COLORS.keys())
        scoreMT = self.allOneScore(typelist,parameterlist,smoothPC,logPC,datatype=datatype,gt=gt)
        nScore = len(typelist)

        plt.figure(figsize=(10,9+nScore*1))
        plt.subplot2grid((5+nScore,11),(0,0),rowspan=5,colspan=10)

        if datatype=="rawhic" and typelist[-1] == "IF":
            self.path = hic2matrix(self.path,self.res,self.chr,gt)
        hp = PlotTAD(self.path,self.res,self.chr,start,end,clmax=clmax)
        if plotTAD == True:
            hp.drawTAD()
        elif plotTAD == False:
            hp.draw()
        for i in range(nScore):
            scoreRegion = scoreMT.iloc[start//self.res:end//self.res,3+i]
            plt.subplot2grid((5+nScore,11),(5+i,0),rowspan=1,colspan=11)
            plt.plot(scoreRegion,c=cols[i],label=scoreRegion.name)
            plt.legend(loc="upper left")
            plt.xlim(start//self.res,end//self.res)
            ticks_pos = np.arange(hp.sbin,hp.ebin+1,(hp.ebin-hp.sbin)/5)
            if i < nScore-1:
                plt.xticks(ticks_pos,[])
            else:
                plt.xticks(ticks_pos,hp.mark)

        return(scoreMT)

    def obtainTwoScore(self,mode,parameter,smoothPC=True,logPC=False,normIF=True,gt=None,datatype="matrix"):
        if datatype == "rawhic" and mode != "IFC":
            self.path = hic2matrix(self.path,self.res,self.chr,gt)
            self.control_path = hic2matrix(self.control_path,self.res,self.chr,gt)

        if mode == "ISC":
            if not parameter: parameter=300000
            score = TADScoreChange(self.path,self.control_path,self.res,self.chr).getChange("IS",int(parameter))
        elif mode == "CIC":
            if not parameter: parameter=300000
            score = TADScoreChange(self.path,self.control_path,self.res,self.chr).getChange("CI",int(parameter))
        elif mode == "DIC":
            if not parameter: parameter=1000000
            score = TADScoreChange(self.path,self.control_path,self.res,self.chr).getChange("DI",int(parameter))
        elif mode == "SSC":
            if not parameter: parameter=300000
            score = TADScoreChange(self.path,self.control_path,self.res,self.chr).getChange("TADss",int(parameter))
        elif mode == "deltaDLR":
            if not parameter: parameter=3000000
            score = deltaDLR(self.path,self.control_path,self.res,self.chr,sizeDLR=int(parameter)).getDeltaDLR()
        elif mode == "IASC":
            if not parameter: parameter=300000
            score = intraScoreChange(self.path,self.control_path,self.res,self.chr,IS_size=int(parameter)).getIntraSC()
        elif mode == "IESC":
            if not parameter: parameter=300000
            score = interScoreChange(self.path,self.control_path,self.res,self.chr,IS_size=int(parameter)).getInterSC()
        elif mode == "DRF":
            if not parameter: parameter=[200000,5000000]
            score = DirectionalRelativeFreq(self.path,self.control_path,self.res,self.chr,
                                            start_distance=int(parameter[0]), end_distance=int(parameter[1])).getDRF()
        elif mode == "CD":
            if not parameter: parameter="pearson"
            score = CorrelationDifference(self.path,self.control_path,self.res,self.chr,method=parameter).getCorrD()
        elif mode == "PC1C":
            score = PC1change(self.path,self.control_path,self.res,self.chr,corr_file=parameter,smoothPC = smoothPC, logPC=logPC).getPC1change()
        elif mode == "IFC":
            if not gt: raise ValueError("Genometable is required for the calculation of IF")
            if datatype=="matrix": raise ValueError("Calculation of IF require rawhic not matrix")
            score = InteractionFrequencyChange(self.path,self.control_path,self.res,self.chr,gt=gt,datatype=datatype,normIF=normIF).getIFC()
        elif mode == "IFCback":
            scoreTreat = pd.read_csv(parameter[0],sep="\t",header=None)
            scoreControl= pd.read_csv(parameter[1],sep="\t",header=None)
            treat = scoreTreat[scoreTreat[0]==self.chr]
            control = scoreControl[scoreControl[0]==self.chr]
            treatlog = np.log1p(treat.iloc[:,3])
            controllog = np.log1p(control.iloc[:,3])
            t = treatlog / np.mean(treatlog[treatlog>0])
            c = controllog / np.mean(controllog[controllog>0])
            score = pd.DataFrame({"chr":treat[0],"start":treat[1],
                                "end":treat[2],"InteractionFrequency Change":t-c})
            score.index = range(score.shape[0])
        else: print("Error: Please specify the correct mode")

        return(score,self.path,self.control_path)

    def allTwoScore(self,typelist=["ISC","CIC","DIC","SSC","deltaDLR","intraSC","interSC","DRF","CorrD","PC1C"],
                    parameterlist=[300000,300000,1000000,300000,3000000,300000,300000,[200000,5000000],"pearson","NotSpecified"],
                    smoothPC=True,logPC=False,datatype="matrix",gt=None):
        for i,type in enumerate(typelist):
            if i == 0:
                multiType,_,_ = self.obtainTwoScore(mode=typelist[i],parameter=parameterlist[i],smoothPC=smoothPC,logPC=logPC,
                                                    datatype=datatype,gt=gt)
            else:
                if datatype=="rawhic" and self.path !=self.rawpath:
                    datatype2="matrix"
                else: datatype2 =datatype
                if typelist[i] == "IFC":
                    self.path = self.rawpath
                    self.control_path = self.rawcontrol
                    datatype2="rawhic"
                next = self.obtainTwoScore(mode=typelist[i],parameter=parameterlist[i],smoothPC=smoothPC,logPC=logPC,
                                            datatype=datatype2,gt=gt)[0].iloc[:,3]
                multiType = pd.concat([multiType,next],axis=1)

        return(multiType)

    def plotTwoScore(self,typelist=["ISC","CIC","DIC","TADssC","deltaDLR","intraSC","interSC","DRF","CorrD","PC1C"],
                    parameterlist=[300000,300000,1000000,300000,3000000,300000,300000,[200000,5000000],"pearson","NotSpecified"],
                    start=0,end=0,clmax=2,plotTAD=False,smoothPC=True,logPC=False,
                    outname="default",datatype="matrix",gt=None):
        import matplotlib.colors as mcolors
        from .callDirectionalTAD import PlotTAD

        cols = list(mcolors.TABLEAU_COLORS.keys())
        cols.remove("tab:gray")
        cols.remove("tab:pink")
        cols.append("black")

        scoreMT = self.allTwoScore(typelist,parameterlist,smoothPC,logPC,datatype=datatype,gt=gt)
        print(scoreMT)
        nScore = len(typelist)

        plt.figure(figsize=(10,9+nScore*1))
        plt.subplot2grid((5+nScore,11),(0,0),rowspan=5,colspan=10)

        if datatype=="rawhic" and typelist[-1] == "IFC":
            self.path = hic2matrix(self.path,self.res,self.chr,gt)
            self.control_path = hic2matrix(self.control_path,self.res,self.chr,gt)

        hp = DiffDraw(self.path,self.control_path,self.res,startSite=start,endSite=end,clmax=clmax)
        if plotTAD == True:
            hp.drawTAD()
        elif plotTAD == False:
            hp.draw_tri()
        for i in range(nScore):
            scoreRegion = scoreMT.iloc[start//self.res:end//self.res+1,3+i]
            plt.subplot2grid((5+nScore,11),(5+i,0),rowspan=1,colspan=11)
            plt.plot(scoreRegion,c=cols[i],label=scoreRegion.name)
            plt.legend(loc="upper left")
            plt.xlim(start//self.res,end//self.res)

            if scoreRegion.name != "CorrelationDifference":
                if scoreRegion.name == "DirectionalRelativeFrequency":
                    posColor = '#e9a3c9'; negColor = '#a1d76a'; opacity=1
                else: posColor = cols[i]; negColor = 'grey'; opacity=0.5

                plt.plot([hp.sbin,hp.ebin],[0,0],"k--",linewidth=0.6)
                plt.fill_between(np.arange(hp.sbin,hp.ebin+1,1),scoreRegion, 0,\
                                where = scoreRegion <=0,facecolor=negColor, alpha=opacity)
                plt.fill_between(np.arange(hp.sbin,hp.ebin+1,1),scoreRegion, 0,\
                                where = scoreRegion >=0,facecolor=posColor, alpha=opacity)

            ticks_pos = np.arange(hp.sbin,hp.ebin+1,(hp.ebin-hp.sbin)/5)
            if i < nScore-1:
                plt.xticks(ticks_pos,[])
            else:
                plt.xticks(ticks_pos,hp.mark)
        return(scoreMT)

class metricHMM:
    def __init__(self,df,ncluster,nRun=10,covMethod= "spherical",random_state=None,HMMtype="Gaussian"):
        if len(df.shape) == 1: df= pd.DataFrame(df)
        self.rawdf = df
        self.label = df.columns
        self.df = pd.DataFrame(np.nan_to_num(df))
        self.nRun = nRun
        self.ncluster = ncluster
        self.state = ["state" + str(i) for i in range(ncluster)]
        self.index = df.index
        self.covMethod = covMethod
        self.limx = [min(df.index),max(df.index)]

        scorelist = []
        modellist = []
        if HMMtype == "Gaussian":
            hmmTrain = hmm.GaussianHMM
        elif HMMtype == "GMM":
            hmmTrain = hmm.GMMHMM

        for i in range(self.nRun):
            model = hmmTrain(n_components=self.ncluster, n_iter=10000,random_state=random_state,
                                    covariance_type=self.covMethod).fit(self.df)
            scorelist.append(model.score(self.df))
            modellist.append(model)
        self.bestmodel = modellist[np.argmax(scorelist)]

    def oneSampleMultiMetric(self,outtype="predict"):
        predictMT = self.bestmodel.predict(self.df)
        emissionMT = pd.DataFrame(self.bestmodel.means_,columns=self.label,index=self.state)
        transitionMT = pd.DataFrame(self.bestmodel.transmat_,columns=self.state,index=self.state)

        if outtype == "predict":
            return(predictMT)
        elif outtype == "emission":
            return(emissionMT)
        elif outtype == "transition":
            return(transitionMT)

    def plotHMM(self,outtype="predict",norm="local"):
        mt = self.oneSampleMultiMetric(outtype)
        if outtype == "predict":
            plt.scatter(self.index,mt,c=mt,marker="8")
            plt.ylim(-1,self.ncluster)
            plt.yticks(range(self.ncluster),self.state)
        elif outtype == "emission":
            if norm == "local":
                zMT = mt.apply(stats.zscore)
            elif norm == "overall":
                u = self.rawdf.mean(axis=0)
                uMT = pd.DataFrame([u]*self.ncluster,index=mt.index)
                sd = self.rawdf.std(axis=0)
                sdMT = pd.DataFrame([sd]*self.ncluster,index=mt.index)
                zMT = (mt - uMT)/sdMT
            sns.heatmap(zMT.T,cmap="coolwarm",vmax=2,vmin=-2)
        elif outtype == "transition":
            sns.heatmap(mt,cmap="coolwarm")

    def plotHiC(self,mode,path,resolution,startSite,endSite,control_path="",clmax=50):
        mt = self.oneSampleMultiMetric("predict")
        plt.figure(figsize=(10,10))
        plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
        if mode == "single":
            hicplot = PlotTri(path,resolution,startSite,endSite,clmax=clmax)
            hicplot.draw()
        elif mode == "differ":
            hicplot = DiffDraw(path,control_path,resolution,startSite,endSite)
            hicplot.draw_tri()
        plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
        plt.scatter(self.index,mt,c=mt,marker="8")
        plt.yticks(range(self.ncluster),self.state)
        plt.xlim(self.limx[0],self.limx[1])

    def oneSampleOneMetric(self):
        pass
    def multiSampleOneMetric(self):
        pass
    def multiSampleMultiMetric(self):
        pass



import argparse
from .MultiTypeScore import *
from .plotMetrics import *
from .plotTwoSample import *
from .MultiSampleScore import *
from .callDirectionalTAD import *
from .calldTADAllchr import *
from .discrete import *
from .stripe import *

def CLI():
    parser = argparse.ArgumentParser(description="HiC1Dmetrics is python3-based tools to \
                                                calculate, visualize, and analyze 1D metrics for Hi-C data \n \
                                                (https://github.com/wangjk321/HiC1Dmetrics) ")
    #parser.set_defaults(func=lambda args: parser.print_help())
    subparsers = parser.add_subparsers(help="Choose the mode to use sub-commands")

    #Function 1
    def func_basic(args):
        args.matrix = args.data
        if args.mode == "plot":
            if args.datatype == "rawhic":
                path = hic2matrix(args.matrix,args.resolution,args.chromosome,args.gt)
                if args.controlmatrix: controlpath = hic2matrix(args.controlmatrix,args.resolution,args.chromosome,args.gt)
            else:
                path = args.matrix
                controlpath = args.controlmatrix

            if args.plottype == "tri":
                if not controlpath:
                    PlotTri(path,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).draw()
                elif controlpath:
                    DiffDraw(path,controlpath,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).draw_tri()
            elif args.plottype == "square":
                if not controlpath:
                    PlotSquare(path,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).draw()
                elif controlpath:
                    DiffDraw(path,controlpath,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).draw_square()
            elif args.plottype == "tad":
                if not controlpath:
                    PlotTAD(path,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).drawTAD(squareSize=int(args.parameter))
                elif controlpath:
                    DiffDraw(path,controlpath,args.resolution,args.chromosome,startSite=args.start,endSite=args.end).drawTAD(squareSize=int(args.parameter))
            plt.savefig(args.outname+".pdf")
        elif args.mode == "dump":
            if args.chromosome == "all":
                allJuicer(args.data,args.normalize,args.resolution,args.gt,args.outname,
                        maxchr=args.maxchr,num=args.nProcesser)
                exit(0)

            if args.datatype not in ["rawhic"] or not args.gt:
                print("Error: dump requires rawhic file and genome_table file"); exit(1)
            codepath = os.path.dirname(os.path.realpath(__file__))
            makeIntra = codepath+"/extract/makeMatrixIntra.sh"
            juicer = codepath+"/jc/jctool_1.11.04.jar"
            foldername = args.outname
            os.system("bash "+makeIntra+" "+args.normalize+" "+"."+" "+args.matrix+" "+
                    str(args.resolution)+" "+args.gt+" "+juicer+" "+args.chromosome+" "+foldername)
        elif args.mode == "gd":
            codepath = os.path.dirname(os.path.realpath(__file__))
            gdcode = codepath+"/gd/makeDensity.sh"
            os.system("sh "+gdcode+" -r "+str(args.resolution)+" -g "+args.data+" -t "+args.chromosome+" -o "+args.outname)
        else:
            print("Unsupported mode"); exit(1)
    parser_basic = subparsers.add_parser("basic",help="Provide basic functions to visualize and handle Hi-C data.")
    parser_basic.add_argument('mode', type=str, help='Type of 1D metrics,,should be one of {dump,plot}')
    parser_basic.add_argument('data', type=str, help='Path of matrix file from JuicerResult')
    parser_basic.add_argument('resolution', type=int,help="Resolution of input matrix")
    parser_basic.add_argument("chromosome",type=str,help="Chromosome number.")
    parser_basic.add_argument("-o","--outname",help="output name",type=str,default="unname")
    parser_basic.add_argument('-c','--controlmatrix', type=str, help='Path of control matrix file from JuicerResult',default=None)
    parser_basic.add_argument('--datatype',type=str,help="matrix or rawhic",default="matrix")
    parser_basic.add_argument('--gt',type=str,help="genome table",default="")
    parser_basic.add_argument('--plottype',type=str,help="Type of plot, could be one of {tri,square,tad}",default="tri")
    parser_basic.add_argument('-s','--start',type=int,help="Start sites for plotting",default=0)
    parser_basic.add_argument('-e','--end',type=int,help="End sites for plotting",default=0)
    parser_basic.add_argument("-p","--parameter",type=str,help="Parameter for indicated metrics",default=None)
    parser_basic.add_argument("--normalize",type=str,help="Normalize methods {NONE/VC/VC_SQRT/KR}",default="KR")
    parser_basic.add_argument("-n","--nProcesser",type=int,help="Number of processors",default=10)
    parser_basic.add_argument('--maxchr',type=int,help="Maximum index of chromosome (human genome is 22,i.e.)",default=None)
    parser_basic.set_defaults(func=func_basic)

    #Function 2
    #=============================================================================
    def func_one(args):
        args.matrix = args.data
        if args.chromosome == "all":
            if not args.maxchr: print("Please sepcify the maximum chromosome"); exit(1)
            if not os.path.exists(args.data):
                print("path not exist"); exit(1)
            scoreAll =oneScoreAllchr(args.data,args.resolution,args.type,args.parameter,
                                    maxchr=args.maxchr,prefix=args.prefix,num=args.nProcesser)
            scoreAll.to_csv(args.outname+"_"+args.type+"_allchr.csv",sep="\t",index=False)
            exit(0)

        if args.parameter and args.type not in ["PC1","IF"]:
            args.parameter = int(args.parameter)

        score = multiScore(args.matrix,args.resolution,args.chromosome,
                            ).obtainOneScore(args.type,parameter=args.parameter,datatype=args.datatype,gt=args.gt,TADfile=args.TADfile,msi=args.msi)
        score.to_csv(args.outname + ".bedGraph", sep="\t", header=False, index=False)

        if args.draw:
            if args.chromosome=="all": print("Error: not supported"); exit(1)
            if args.type == "IF":
                args.parameter = args.outname + ".bedGraph"
            print("==========output figure==========")
            PlotBedGraph(args.matrix,args.resolution,args.chromosome,startSite=args.start,
                        endSite=args.end,datatype=args.datatype,
                        gt=args.gt).draw(args.type,UniqueParameter=args.parameter)
            plt.savefig(args.outname+".pdf")


    parser_one = subparsers.add_parser("one",help="1D metrics designed for one Hi-C sample.",
                                            description="1D metrics designed for one Hi-C sample.")
    parser_one.add_argument('type', type=str, help='Type of 1D metrics,,should be one of {IS,CI,DI,SS,DLR,PC1,IES,IAS,IF}.')
    parser_one.add_argument('data', type=str, help='Path of matrix or rawhic file.')
    parser_one.add_argument('resolution', type=int,help="Resolution of input matrix.")
    parser_one.add_argument("chromosome",type=str,help="Chromosome number.")
    parser_one.add_argument("-p","--parameter",type=str,help="Parameter for indicated metrics.",default=None)
    parser_one.add_argument("-o","--outname",help="output name (default: 'metrics').",type=str,default="unname")
    parser_one.add_argument("-d","--draw",action='store_true',help="Plot figure for candidate region.",default=False)
    parser_one.add_argument("-s",'--start',type=int,help="Start sites for plotting.",default=0)
    parser_one.add_argument("-e",'--end',type=int,help="End sites for plotting.",default=0)
    parser_one.add_argument('--datatype',type=str,help="Type of input data: [matrix(default),rawhic,cool].",default="matrix")
    parser_one.add_argument('--msi',type=str,help="Method for significant interactions: [fithic2,hiccups]",default="fithic2")
    parser_one.add_argument('--gt',type=str,help="genome_table file.",default="")
    parser_one.add_argument('--prefix',type=str,help="${prefix}chr1.matrix.gz",default="observed.KR.")
    parser_one.add_argument('--maxchr',type=int,help="Maximum index of chromosome (human genome is 22,i.e.)",default=None)
    parser_one.add_argument("-n","--nProcesser",type=int,help="Number of processors",default=10)
    parser_one.add_argument("-t","--TADfile",type=str,help="Give a TAD file, instead of using building-in TAD calling method",default=None)
    parser_one.set_defaults(func=func_one)

    #Function 3
    #=============================================================================
    def func_two(args):
        args.matrix = args.data
        args.controlmatrix = args.controldata

        if args.chromosome == "all":
            if not args.maxchr: print("Please sepcify the maximum chromosome"); exit(1)
            if not os.path.exists(args.data):
                print("path not exist"); exit(1)
            scoreAll =twoScoreAllchr(args.data,args.controldata,args.resolution,args.type,args.parameter,
                                    maxchr=args.maxchr,prefix=args.prefix)
            scoreAll.to_csv(args.outname+"_"+args.type+"_allchr.csv",sep="\t",index=False)
            exit(0)

        ms = multiScore(args.matrix,args.resolution,args.chromosome,control_path=args.controlmatrix)
        score,path,control_path = ms.obtainTwoScore(args.type,parameter=args.parameter,datatype=args.datatype,gt=args.gt)
        score.to_csv(args.outname + ".bedGraph", sep="\t", header=False, index=False)

        if args.draw:
            if args.type != "IFC":
                DiffDraw(path,control_path,args.resolution,chr = args.chromosome,startSite=args.start,
                        endSite=args.end,datatype="matrix",gt=args.gt).drawMetric("custom",customfile=args.outname + ".bedGraph",name=args.type)
            elif args.type == "IFC":
                DiffDraw(path,control_path,args.resolution,chr = args.chromosome,startSite=args.start,
                        endSite=args.end,datatype="rawhic",gt=args.gt).drawMetric("custom",customfile=args.outname + ".bedGraph",name=args.type)
            plt.savefig(args.outname+".pdf")

        os.system("rm -rf MatrixTemp*")

    parser_two = subparsers.add_parser("two",help="1D metrics designed for comparison of two Hi-C samples",
                                            description="1D metrics designed for comparison of two Hi-C samples")
    parser_two.add_argument('type', type=str, help='Type of 1D metrics for two-sample comparison,should be one of {ISC,CIC,SSC,deltaDLR,CD,IESC,IASC,IFC,DRF}')
    parser_two.add_argument('data', type=str, help='Path of treated file (matrix or rawhic).')
    parser_two.add_argument('controldata', type=str, help='Path of control file (matrix or rawhic).')
    parser_two.add_argument('resolution', type=int,help="Resolution of input matrix")
    parser_two.add_argument("chromosome",type=str,help="Chromosome number ('chr21',i.e).")
    parser_two.add_argument("-p","--parameter",type=str,help="Parameter for indicated metrics",default=None)
    parser_two.add_argument("-o","--outname",help="output name (default: metricsChange)",type=str,default="metricsChange")
    parser_two.add_argument("-d","--draw",action='store_true',help="Plot figure for candidate region",default=False)
    parser_two.add_argument('-s','--start',type=int,help="Start sites for plotting",default=0)
    parser_two.add_argument('-e','--end',type=int,help="End sites for plotting",default=0)
    parser_two.add_argument('--datatype',type=str,help="matrix or rawhic",default="matrix")
    parser_two.add_argument('--gt',type=str,help="genome table file",default="")
    parser_two.add_argument('--prefix',type=str,help="${prefix}chr1.matrix.gz",default="observed.KR.")
    parser_two.add_argument('--maxchr',type=int,help="Maximum index of chromosome (human genome is 22,i.e.)",default=None)
    parser_two.add_argument("-n","--nProcesser",type=int,help="Number of processors",default=10)
    parser_two.set_defaults(func=func_two)

    #Function 4
    #=============================================================================
    def func_types(args):
        args.matrix = args.data
        typelist = args.typelist.split(",")
        parameterlist = args.parameter.split(",")
        if not args.controlmatrix:
            if not set(typelist).issubset(["IS","CI","DI","SS","DLR","PC1","IES","IAS","IF"]):
                print("Error: not supported"); exit(1)
            if "IF" in typelist and args.datatype == "matrix": print("Error: IF required rawhic datatype"); exit(1)
            ms = multiScore(args.matrix,args.resolution,args.chromosome)
            if not args.draw:
                score = ms.allOneScore(typelist,parameterlist,datatype=args.datatype,gt=args.gt)
            elif args.draw:
                score = ms.plotOneScore(typelist,parameterlist,datatype=args.datatype,gt=args.gt,start=args.start,end=args.end)
                plt.savefig(args.outname+".pdf")
            print(score.iloc[550:650,:])
            score.to_csv(args.outname + ".csv", sep="\t", header=True, index=False)
        elif args.controlmatrix:
            if not set(typelist).issubset(["ISC","CIC","SSC","deltaDLR","CD","IESC","IASC","IFC","DRF"]):
                print("Error: not supported"); exit(1)
            if "IFC" in typelist and args.datatype == "matrix": print("Error: IFC required rawhic datatype"); exit(1)
            if "DRF" in typelist:
                DRFpos = typelist.index("DRF")
                parameterlist[DRFpos] = parameterlist[DRFpos].split("-")
                print(parameterlist)
            ms = multiScore(args.matrix,args.resolution,args.chromosome,control_path=args.controlmatrix)
            if not args.draw:
                score = ms.allTwoScore(typelist,parameterlist,datatype=args.datatype,gt=args.gt)
            elif args.draw:
                score = ms.plotTwoScore(typelist,parameterlist,datatype=args.datatype,gt=args.gt,start=args.start,end=args.end)
                plt.savefig(args.outname+".pdf")
            print(score.iloc[550:650,:])
            score.to_csv(args.outname + ".csv", sep="\t", header=True, index=False)

        os.system("rm -rf MatrixTemp*")

    parser_types = subparsers.add_parser("multitypes",help="Various types of 1D metrics for the same sample",
                                            description="Various types of 1D metrics for the same sample")
    parser_types.add_argument('typelist', type=str, help='Type of 1D metrics,should be {IS,CI,DI,SS,DLR,PC1,IES,IAS,IF} or {ISC,CIC,SSC,deltaDLR,CD,IESC,IASC,IFC,DRF}')
    parser_types.add_argument('data', type=str, help='Path of matrix file or raw .hic file')
    parser_types.add_argument('resolution', type=int,help="Resolution of input matrix")
    parser_types.add_argument("chromosome",type=str,help="Chromosome number.")
    parser_types.add_argument("-p","--parameter",type=str,help="Parameter for indicated metrics",default=None,required=True)
    parser_types.add_argument('-c','--controlmatrix', type=str, help='Path of control matrix file from JuicerResult',default=None)
    parser_types.add_argument("-o","--outname",help="output name (default metrics)",type=str,default="multitypes_metrics")
    parser_types.add_argument('--datatype',type=str,help="matrix or rawhic",default="matrix")
    parser_types.add_argument('--gt',type=str,help="genome table",default="")
    parser_types.add_argument("-d","--draw",action='store_true',help="Plot figure for candidate region",default=False)
    parser_types.add_argument('-s','--start',type=int,help="Start sites for plotting",default=0)
    parser_types.add_argument('-e','--end',type=int,help="End sites for plotting",default=0)
    parser_types.set_defaults(func=func_types)

    #Function 5
    #=============================================================================
    def func_samples(args):
        if args.type == "IF" and args.datatype == "matrix": print("Error: IF required rawhic datatype"); exit(1)
        datafile = pd.read_csv(args.data,sep="\t",header=None)
        labels = list(datafile.iloc[:,0])
        samplelist = list(datafile.iloc[:,1])
        if not args.corr and not args.heat and not args.line and not args.discrete and not args.anova:
            score = getMultiSamplesScore(samplelist,labels,args.resolution,args.chromosome,args.type,args.parameter,
                                        datatype=args.datatype,gt=args.gt)
        elif args.corr:
            ms = repQC(samplelist,labels,args.resolution,args.chromosome,args.type,args.parameter,datatype=args.datatype,gt=args.gt)
            score = ms.score
            ms.corr_plot()
            plt.savefig(args.outname+"_corr.pdf")
        elif args.discrete:
            if args.datatype == "rawhic":
                print("not supported");exit(1)
            msd = multiSampleDiscrete(samplelist,labels,args.resolution,args.chromosome,args.type,args.parameter)
            plotpath= samplelist[0]
            msd.plotMultiDiscrete(plotpath,args.start,args.end)
            plt.savefig(args.outname+"_discrete.pdf")
            exit(0)
        elif args.anova:
            if args.end == 0:
                print("Please specify a region to calculate p-value");exit(1)
            ms = repQC(samplelist,labels,args.resolution,args.chromosome,args.type,args.parameter,datatype=args.datatype,gt=args.gt)
            score = ms.score
            plist,qlist = ms.anova_like(args.start,args.end)
            df = pd.DataFrame()
            df["chr"] = [args.chromosome]* len(range(args.start,args.end,args.resolution))
            df["start"] = list(range(args.start,args.end,args.resolution))
            df["end"] = df["start"] + args.resolution
            df['pvalue'] = plist
            df['qvalue'] = qlist
            df.to_csv(args.outname+"_anova.txt",sep="\t",index=False)

        elif args.line or args.heat:
            ms = repQC(samplelist,labels,args.resolution,args.chromosome,args.type,args.parameter,
                        datatype=args.datatype,gt=args.gt)
            score = ms.score
            if args.datatype == "matrix":
                plotpath= samplelist[0]
            elif args.datatype == "rawhic":
                plotpath = hic2matrix(samplelist[0],args.resolution,args.chromosome,args.gt)

            if args.heat: plottype = "heat"
            elif args.line: plottype = "line"

            ms.heatmap_tri(plotpath,args.start,args.end,clmax=args.clmax,heatmin=None,plottype=plottype)
            plt.savefig(args.outname+"_"+plottype+".pdf")

        score.to_csv(args.outname + ".csv", sep="\t", header=True, index=False)

        print(score.iloc[550:650,:])
        os.system("rm -rf MatrixTemp*")

    parser_samples = subparsers.add_parser("multisamples",help="The same metrics for muliple samples",
                                            description="The same metrics for muliple samples")
    parser_samples.add_argument('type', type=str, help='Type of 1D metrics,,should be one of {IS,CI,DI,SS,DLR,PC1,IES,IAS,IF} or {ISC,CIC,SSC,deltaDLR,CD,IESC,IASC,IFC,DRF}')
    parser_samples.add_argument('data',type=str,help="a txt contain paths for all samples")
    parser_samples.add_argument('resolution', type=int,help="Resolution of input matrix")
    parser_samples.add_argument("chromosome",type=str,help="Chromosome number.")
    parser_samples.add_argument('--datatype',type=str,help="matrix or rawhic",default="matrix")
    parser_samples.add_argument('--samplelist', type=str, help='list of file path, can be rawhic or matrix')
    parser_samples.add_argument('--labels', type=str, help='list of file name')
    parser_samples.add_argument("-p","--parameter",type=str,help="Parameter for indicated metrics",default=None)
    parser_samples.add_argument("-o","--outname",help="output name (default metrics)",type=str,default="multisamples_metrics")
    parser_samples.add_argument('--gt',type=str,help="genome table",default="")
    parser_samples.add_argument("--corr",action='store_true',help="Plot correlation for all samples",default=False)
    parser_samples.add_argument("--heat",action='store_true',help="Plot raw heatmap for all samples",default=False)
    parser_samples.add_argument("--line",action='store_true',help="Plot line chart for all samples",default=False)
    parser_samples.add_argument("--anova",action='store_true',help="Plot line chart for all samples",default=False)
    parser_samples.add_argument("--discrete",action='store_true',help="Plot discrete heatmap for all samples",default=False)
    parser_samples.add_argument('-s','--start',type=int,help="Start sites for plotting",default=0)
    parser_samples.add_argument('-e','--end',type=int,help="End sites for plotting",default=0)
    parser_samples.add_argument('--clmax',type=int,help="End sites for plotting",default=None)

    parser_samples.set_defaults(func=func_samples)

    #Function 6
    #=============================================================================
    def func_call(args):
        args.matrix = args.data
        if args.datatype == "rawhic" and args.mode != "hubs":
            path = hic2matrix(args.matrix,args.resolution,args.chromosome,args.gt)
            if args.controlmatrix: controlpath = hic2matrix(args.controlmatrix,args.resolution,args.chromosome,args.gt)
        else:
            path = args.matrix
            controlpath = args.controlmatrix

        if args.mode == "TAD":
            if not args.parameter: args.parameter = 300000
            tad = TADcallIS(path,args.resolution,args.chromosome,squareSize=int(args.parameter))
            tad.to_csv(args.outname + "_TAD.csv", sep="\t", header=True, index=False)
        elif args.mode == "dTAD":
            if args.parameter:
                parameter = args.parameter.split("-")
            else:
                parameter = [200000,5000000]
            if not args.controlmatrix: print("Error: DRF requires control data"); exit(1)
            dt = DirectionalTAD(path,controlpath,args.resolution,args.chromosome,
                                startDRF=int(parameter[0]), sizeDRF=int(parameter[1]))
            leftdTAD,rightdTAD,_ = dt.extractRegion()
            leftdTAD.to_csv(args.outname + "_leftdTAD.csv", sep="\t", header=True, index=False)
            rightdTAD.to_csv(args.outname + "_rightdTAD.csv", sep="\t", header=True, index=False)
        elif args.mode == "stripeTAD":
            st = stripeTAD(path,args.resolution,args.chromosome)
            stripeTAD = st.callStripe(squareSize=300000)
            stripeTAD.to_csv(args.outname + "_stripeTAD.csv", sep="\t", header=True, index=False)
        elif args.mode == "stripe":
            st = call_stripe(path,args.resolution,args.chromosome)
            stripe = st.callStripe(squareSize=300000,strong_thresh=0.2)
            stripe.to_csv(args.outname + "_stripes.csv", sep="\t", header=True, index=False)
        elif args.mode == "hubs":
            if args.datatype != "rawhic": print("Error: hubs requires rawhic datatype"); exit(1)
            IF = InteractionFrequency(path,args.resolution,args.chromosome,gt=args.gt).getIF()
            thresh = np.percentile(IF.iloc[:,3],90)
            hubregion = IF[IF.iloc[:,3]>thresh]
            hubregion.to_csv(args.outname + "_hubs_IF.csv", sep="\t", header=True, index=False)
            os.system("sed '1d' " + args.outname + "_hubs_IF.csv" + " |bedtools merge -i stdin > "+ args.outname + "_hubs.csv")
        else:
            print("unsupported model");exit(1)

    parser_call = subparsers.add_parser("call",help="Extract secondary information from metrics (dTAD, stripeTAD, et.al)",
                                            description="Extract secondary information from metrics (dTAD, stripeTAD, et.al)")
    parser_call.add_argument('mode', type=str, help='Running mode,,should be one of {dTAD,stripe,stripeTAD,TAD,hubs}')
    parser_call.add_argument('data', type=str, help='Path of matrix file or raw .hic file')
    parser_call.add_argument('resolution', type=int,help="Resolution of input matrix")
    parser_call.add_argument("chromosome",type=str,help="Chromosome number.")
    parser_call.add_argument("-o","--outname",help="output name",type=str,default="defaultname")
    parser_call.add_argument('-c','--controlmatrix', type=str, help='Path of control matrix file from JuicerResult',default=None)
    parser_call.add_argument('--datatype',type=str,help="matrix or rawhic",default="matrix")
    parser_call.add_argument('--gt',type=str,help="genome table",default="")
    parser_call.add_argument("-p","--parameter",type=str,help="Parameter for indicated mode",default=None)
    parser_call.set_defaults(func=func_call)

    parser.add_argument("-V","--version",help="Show h1d version",action='store_true',default=False)
    args = parser.parse_args()
    if args.version:
        print("h1d version 0.1.13")
        exit(0)
    try:
        func = args.func
    except AttributeError:
        parser.error("too few arguments")
    func(args)
    os.system("rm -rf MatrixTemp*")
    os.system("rm -rf info.txt")

if __name__ == '__main__':
    CLI()



from calculateMetrics import *

class DiffCI(BasePara):
    #compare two metrices need normalization
    def __init__(self,path,control_path,resolution,chromosome,out_name="DRF",
                diffCI_size=300000):
        super().__init__(path,resolution,chromosome,out_name)
        self.path = path
        self.control_path = control_path
        treat = loadWithNorm(path,log = True).values
        control = loadWithNorm(control_path,log = True).values
        smooth = 2
        self.treat = ndimage.median_filter(treat,smooth)
        self.control = ndimage.median_filter(control,smooth)
        self.matrix = self.treat-self.control
        self.diffCI_size = diffCI_size

        if self.treat.shape[0] != self.control.shape[0]:
            print("Error: Input/control matrix have different shape")
            exit(1)

    def getDiffCI(self):
        CI_Bin = round(self.diffCI_size/self.resolution)

        array = np.zeros(self.matrix_shape)
        for i in range(self.matrix_shape):
            if(i - CI_Bin < 0 or i + CI_Bin >= self.matrix_shape): continue
            matA = self.matrix[i-CI_Bin:i,i-CI_Bin:i]
            matB = self.matrix[i+1:i+CI_Bin+1,i+1:i+CI_Bin+1]
            A = np.triu(matA,1).sum()
            B = np.triu(matB,1).sum()
            C = self.matrix[i-CI_Bin: i, i+1: i+CI_Bin+1].sum()
            if np.isnan(A) or np.isnan(B) or np.isnan(C): continue  #skip NaN value
            S = 2*C-(A+B)
            if S < 0:
                array[i] = -np.log1p(-S)
            else:
                array[i] = np.log1p(S)

        return super().makeDF(array,"diffCI")

    #def getCSV(self):
    #    super().makeCSV(self.getDRF())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from plotMetrics import *
from calculateMetrics import *

if __name__ == '__main__':
#def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("mode",help="Type of hic metrics,should be one of IS,CI,DI,DFR",type=str)
    parser.add_argument("matrix",help="Path of matrix file from JuicerResult",type=str)
    parser.add_argument("resolution",help="Resolution of input matrix",type=int)
    parser.add_argument("chromosome",help="chromosome No.",type=str)

    parser.add_argument("--square_size",help="IS parameter, Square size of Insulation Score (default 150000)",type=int,default=150000)
    parser.add_argument("--CI_size",help="CI paramater, Square size of Contrast Index (default 150000)",type=int,default=150000)
    parser.add_argument("--DI_distance",help="DI parameter, Distance used to calculate DI (default 1000000)",type=int,default=1000000)
    parser.add_argument("--control_matrix",help="DRF paramater, control matrix",type=str)
    parser.add_argument("--start_distance",help="DRF paramater, start_distance",type=int,default=0)
    parser.add_argument("--end_distance",help="DRF parameter, end_distance",type=int,default=2000000)

    parser.add_argument("-O","--out_name",help="-O or out_name (default noName)",type=str,default="noName")
    parser.add_argument("--plot_from",help="Start site of plot",type=int,default=0)
    parser.add_argument("--plot_to",help="End site of plot",type=int,default=0)

    args = parser.parse_args()

    plot = PlotBedGraph(args.matrix,args.resolution,clmin=0,clmax=20,\
                    title=args.out_name,chr=args.chromosome, \
                    startSite= args.plot_from,endSite=args.plot_to)

    if args.mode == "IS":
        print("--------Insulation Score Mode--------- \n")
        print("--------Reading Files--------- \n")
        IS = InsulationScore(args.matrix,args.resolution,args.chromosome,args.out_name,args.square_size)
        print(IS)
        IS.getCSV()
        print("---------Saving pdf----------")
        plot.makePDF("IS",args.out_name)
    elif args.mode == "CI":
        print("--------Contrast Index Mode--------- \n")
        print("--------Reading Files--------- \n")
        CI = ContrastIndex(args.matrix,args.resolution,args.chromosome,args.out_name,args.CI_size)
        print(CI)
        CI.getCSV()
        print("---------Saving pdf----------")
        plot.makePDF("CI",args.out_name)
    elif args.mode == "DI":
        print("--------Directionality Index Mode--------- \n")
        print("--------Reading Files--------- \n")
        DI = DirectionalityIndex(args.matrix,args.resolution,args.chromosome,args.out_name,args.DI_distance)
        print(DI)
        DI.getCSV()
        print("---------Saving pdf----------")
        plot.makePDF("DI",args.out_name)
    elif args.mode == "DRF":
        print("--------DirectionalRelativeFreq Mode--------- \n")
        print("--------Reading Files--------- \n")
        DRF = DirectionalRelativeFreq(args.matrix,args.control_matrix,args.resolution,args.chromosome, \
                                        args.out_name,args.start_distance,args.end_distance)
        print(DRF)
        DRF.getCSV()
    else:
        print("Please specific the metrics type")
        exit(1)

    print("\n--------Finish--------- \n")



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import sys

def parse_argv():
    usage = 'Usage: \n    python {} <inputfile> <outputfile> <genometable> <chr> <resolution> [--help]'.format(__file__)
    arguments = sys.argv
    if len(arguments) == 1:
        print(usage)
        exit()
    # ファイル自身を指す最初の引数を除去
    arguments.pop(0)
    # - で始まるoption
    options = [option for option in arguments if option.startswith('-')]

    if '-h' in options or '--help' in options:
        print(usage)

    return arguments

#def getfilename(i, j):
#    return dir + "/" + str(res) + "/chr" + str(i) + "-chr" + str(j) + "/" + ntype + ".matrix.gz"

if __name__ == '__main__':
    arguments = parse_argv()
    inputfile = arguments[0]
    outputfile = arguments[1]
    gtfile = arguments[2]
    chr = arguments[3]
    resolution = int(arguments[4])

    genometable = pd.read_csv(gtfile, delimiter='\t', index_col=[0], header=None)
    chrlen = genometable.loc[chr]
    binlen = int(chrlen/resolution) +1

    arr = np.zeros((binlen, binlen))
    #    d = pd.read_csv(inputfile, delimiter='\t', header=None, index_col=[0,1])
    d = pd.read_csv(inputfile, delimiter='\t', header=None)
    d = d.set_index([0,1])
    d = d.iloc[:,0]
    d = d.unstack(fill_value=0)
    index = np.arange(binlen) * resolution
    d = d.reindex(index, columns=index, fill_value=0)
    d.index.name = None
    d.columns.name = None

    triu = np.triu(d)
    array = triu + triu.T - np.diag(np.diag(triu))
    df = pd.DataFrame(array, index=d.index, columns=d.columns)

    df.to_csv(outputfile, sep='\t', compression='gzip')



from .MultiTypeScore import *
from .calculateMetrics import *
import os

def getDiscrete(path,res,chr,mode,parameter,control_path="",label=True):
    if mode == "PC1":  #Acompartment~1,Bcompartment~-1
        ob = multiScore(path,res,chr)
        score = ob.obtainOneScore(mode,parameter)
        if label:
            state = np.array(["NA"]*score.shape[0],dtype=object)
            state[score.iloc[:,3] > 0] = "CompartA"
            state[score.iloc[:,3] < 0] = "CompartB"
        else:
            state = np.array([np.NaN]*score.shape[0])
            state[score.iloc[:,3] > 0] = 1
            state[score.iloc[:,3] < 0] = 0
        score.iloc[:,3] =state

    elif mode in ["deltaDLR"]:
        #positive-decompaction: 1; negative-compaction:-1
        ob = multiScore(path,res,chr,control_path=control_path)
        score = ob.obtainTwoScore(mode,parameter)
        state = np.array(["NA"]*score.shape[0],dtype=object)
        state[score.iloc[:,3] > 0] = "decompact"
        state[score.iloc[:,3] < 0] = "compact"
        score.iloc[:,3] =state

    elif mode in ["ISC","CIC","intraSC","interSC","DRF"]:
        # ISC: positive-more interaction: 1; negative-less interaction:-1
        # CIC: postive 1 stronger boundary. negative -1 weaker boundart
        ob = multiScore(path,res,chr,control_path=control_path)
        score = ob.obtainTwoScore(mode,parameter)
        state = np.array(["NA"]*score.shape[0],dtype=object)
        state[score.iloc[:,3] > 0] = "up"+mode
        state[score.iloc[:,3] < 0] = "down"+mode
        score.iloc[:,3] =state

    elif mode == "CorrD":
        ob = multiScore(path,res,chr,control_path=control_path)
        score = ob.obtainTwoScore(mode,parameter)
        thresh = score.CorrD.describe()[5]
        state = np.array(["NA"]*score.shape[0],dtype=object)
        state[score.iloc[:,3] >= thresh] = str("highCorr")
        state[score.iloc[:,3] < thresh] = str("lowCorr")
        score.iloc[:,3] =state

    elif mode == "PC1C":
        ob = multiScore(path,res,chr,control_path=control_path)
        score = ob.obtainTwoScore(mode,parameter)
        state = np.array(["none"]*score.shape[0],dtype=object)
        state[score.iloc[:,3] > 0] = "BtoA"
        state[score.iloc[:,3] < 0] = "AtoB"
        score.iloc[:,3] =state

    elif mode == "border":
        tad = TADcallIS(path,res,chr)
        bd = np.concatenate([np.array(tad.TADstart),np.array(tad.TADend)])
        bd = np.unique(bd)
        IS = multiScore(path,res,chr).obtainOneScore("IS",parameter)

        state = np.array(["nonBorder"]*IS.shape[0],dtype=object)
        for i in bd:
            state[IS.start == i] = "border"
            state[IS.start == i-res] = "border"
            state[IS.start == i+res] = "border"
        score = IS
        score.iloc[:,3] =state
        score.columns=["chr","start","end","TADborder"]
    else:
        print("Only support PC1|border")
        exit(1)

    return(score)

class multiTypeDiscrete:
    def __init__(self,path,control_path,res,chr,
                typelist=["PC1","border","deltaDLR","ISC","CIC","intraSC","interSC","DRF","CorrD","PC1C"],
                parameterlist=["NoDefault",300000,3000000,300000,300000,300000,300000,[200000,5000000],"pearson","NoDefault"]):
        self.path = path
        self.res = res
        self.chr = chr
        self.control_path = control_path
        self.typelist = typelist
        self.parameterlist = parameterlist

    def multiDiscrete(self):
        for i,type in enumerate(self.typelist):
            print(i,type)
            if i == 0:
                mt = getDiscrete(self.path,self.res,self.chr,self.typelist[i],
                                self.parameterlist[i],control_path=self.control_path)
            else:
                next = getDiscrete(self.path,self.res,self.chr,self.typelist[i],
                                self.parameterlist[i],control_path=self.control_path).iloc[:,3]
                mt = pd.concat([mt,next],axis=1)

        return(mt)

    def makecsv(self,outname="multiDiscrete.txt",s=450,e=650):
        self.multiDiscrete().iloc[s:e,3:].to_csv(outname,sep="\t",index=None)

    def useHMM(self):
        self.makecsv()
        import rpy2.robjects as robjects
        r = robjects.r
        if not os.path.exists("seqHMM"):
            os.mkdir("seqHMM")

        r("library(seqHMM)")
        r("mt <- read.csv('multiDiscrete.txt',sep='\t')")
        r("mt[mt=='NA'] <- NA")

        r("mtlist <- list()")
        r("for(i in seq(1,dim(mt)[2])){mtlist[[i]] <- seqdef(t(as.matrix(mt[,i])))}")

        r("pdf('seqHMM/observedstate.pdf',width=10,height = 8)")
        r("ssplot(mtlist, title = 'Observed states plots')")
        r("dev.off()")

        r("mod <- build_hmm(observations = mtlist, n_states = 5)")
        r("hmm <- fit_model(mod,global_step = TRUE, local_step = TRUE,threads=10,control_em = list(restart=list(times = 100)))")

        r("tMT <- hmm$model$transition_probs")
        r("write.table(tMT,'seqHMM/transitionMatrix.txt',sep='\t')")

        r("hMT <- hidden_paths(hmm$model)")
        r("write.table(hMT,'seqHMM/hiddenState.txt',sep='\t')")

        r("emp<- hmm$model$emission_probs")
        r("eMT <- matrix(data = NA,dim(emp[[1]])[1],length(emp))")
        r("label <- c()")
        r("for(i in seq(1,length(emp))){eMT[,i] <- emp[[i]][,1];label[i] <- colnames(emp[[i]])[1]}")
        r("colnames(eMT) <- label")
        r("row.names(eMT) <- row.names(emp[[1]])")
        r("write.table(eMT,'seqHMM/emissionMatrix.txt',sep='\t')")

        hMT = pd.read_csv("seqHMM/hiddenState.txt",sep="\t")
        eMT = pd.read_csv("seqHMM/emissionMatrix.txt",sep="\t")
        tMT = pd.read_csv("seqHMM/transitionMatrix.txt",sep="\t")

        return(hMT,eMT,tMT)

    def plotHMM(self,type="hidden",plotHiC="",s=None,e=None,clmax=100):
        hMT,eMT,tMT = self.useHMM()
        if type == "hidden":
            from sklearn.preprocessing import LabelEncoder
            hmt = hMT.iloc[0,:]
            le = LabelEncoder().fit(hmt)
            hmtN = le.transform(hmt)
            if plotHiC in ["single","differ"]:
                plt.figure(figsize=(10,10))
                plt.subplot2grid((6,11),(0,0),rowspan=5,colspan=10)
                if plotHiC == "single":
                    hicplot = PlotTri(self.path,self.res,startSite=s,endSite=e,clmax=clmax)
                    hicplot.draw()
                elif plotHiC == "differ":
                    hicplot = DiffDraw(self.path,self.control_path,self.res,startSite=s,endSite=e)
                    hicplot.draw_tri()
                plt.subplot2grid((6,11),(5,0),rowspan=1,colspan=11)
            plt.scatter(range(len(hmt)),hmtN,c=hmtN,marker="8")
            plt.yticks(range(5),list(le.classes_))
        elif type == "transition":
            sns.heatmap(tMT,cmap="coolwarm")
        elif type == "emission":
            sns.heatmap(eMT.T,cmap="coolwarm")


class multiSampleDiscrete:
    def __init__(self,pathlist,namelist,res,chr,mode,UniqueParameter):
        self.pathlist = pathlist
        self.namelist = namelist
        self.res = res
        self.chr = chr
        self.mode = mode
        self.UniqueParameter = UniqueParameter
        self.nScore = len(namelist)

    def getMultiDiscrete(self,label=True):
        for i,path in enumerate(self.pathlist):
            if i==0: metricMT = getDiscrete(path,self.res,self.chr,self.mode,self.UniqueParameter,label=label)
            else:
                next = getDiscrete(path,self.res,self.chr,self.mode,self.UniqueParameter,label=label).iloc[:,3:4]
                metricMT = pd.concat([metricMT,next],axis=1)

        metricMT.index = metricMT.start.tolist()
        metricMT = metricMT.iloc[:,3:]
        metricMT.columns = self.namelist
        return(metricMT)

    def plotMultiDiscrete(self,hic_path,start,end,clmax=100,heatmin=None,interpolation='none'):
        sbin = start//self.res
        ebin = end//self.res

        from .callDirectionalTAD import PlotTAD
        plt.figure(figsize=(10,9+self.nScore))
        plt.subplot2grid((5+int(self.nScore/1.2),11),(0,0),rowspan=5,colspan=10)
        hp = PlotTAD(hic_path,self.res,self.chr,start,end,clmax=clmax)
        hp.draw()

        plt.subplot2grid((5+int(self.nScore/1.2),11),(5,0),rowspan=(self.nScore//7)+1,colspan=11)
        df = self.getMultiDiscrete(label=False).iloc[sbin:ebin,:].T
        plt.imshow(df,aspect="auto",interpolation=interpolation,vmin=heatmin)
        plt.yticks(range(self.nScore),self.namelist)
        plt.xticks(np.arange(0,df.shape[1]+1,(df.shape[1])/5),hp.mark)



import pandas as pd
import numpy as np
import os
import random

def loadDenseMatrix(filename,log=False):
    #print(filename)
    data = pd.read_csv(filename, delimiter='\t', index_col=0)
    if log == True:
        return(np.log1p(data))
    else: return data

def loadWithNorm(filename,method= "RPM",log = False):
    data = pd.read_csv(filename, delimiter='\t', index_col=0)
    if method == "RPM":
        data = (10000000 * data) / np.nansum(data)
    if log:
        return np.log1p(data)
    else:
        return data

def hic2matrix(path,res,chr,gt):
    if not gt:
        print("rawhic require genome_table file");exit(1)
    if not isinstance(pd.read_csv(gt,sep="\t",header=None).iloc[0,1],int):
        print("Wrong genome_table file.")
        print("Please use the check your genome_table file. Is it tab separated ?")
        exit(1)
    codepath = os.path.dirname(os.path.realpath(__file__))
    makeIntra = codepath+"/extract/makeMatrixIntra.sh"
    juicer = codepath+"/jc/jctool_1.11.04.jar"
    foldername = "./MatrixTemp"+str(random.random())
    os.system("bash "+makeIntra+" "+"KR"+" "+"."+" "+path+" "+str(res)+" "+gt+" "+juicer+" "+chr+" "+foldername + "> info.txt")
    matrixpath = foldername+"/"+str(res)+"/observed.KR."+chr+".matrix.gz"
    return(matrixpath)

def cool2matrix(path,res,chr,gt):
    if not gt: print("cool require genome_table file");exit(1)
    codepath = os.path.dirname(os.path.realpath(__file__))
    makeIntra = codepath+"/extract/coolerdump.sh"
    foldername = "./MatrixTemp"+str(random.random())
    os.system("bash "+makeIntra+" "+path+" "+str(res)+" "+chr+" "+gt+" "+foldername + "> info.txt")
    matrixpath = foldername+"/"+str(res)+"/"+chr+".matrix.gz"
    return(matrixpath)



import pandas as pd
import seaborn as sns
import numpy as np
from scipy.stats import pearsonr
import matplotlib.pyplot as plt


e1 = pd.read_csv("../test_data/emissions_8.txt",sep="\t",index_col=0)
e2 = pd.read_csv("../test_data/emissions_9.txt",sep="\t",index_col=0)
e2.columns

e1=e1[e2.columns]
e1.index = ["e1_state"+str(i) for i in range(1,9)]
e2.index = ["e2_state"+str(i) for i in range(1,10)]
e1

corMT = pd.DataFrame(index=["e1_state"+str(i) for i in range(1,9)],
                    columns=["e2_state"+str(i) for i in range(1,10)])
for i in range(8):
    for j in range(9):
        corMT.iloc[i,j] = pearsonr(e1.iloc[i,:],e2.iloc[j,:])[0]
df = corMT.astype(float)

plt.figure(figsize=(9,8))
sns.heatmap(df,cmap="coolwarm")



from .calculateMetrics import *

class DirectionalRelativeFreq(BasePara):
    #compare two metrices need normalization
    def __init__(self,path,control_path,resolution,chromosome,out_name="DRF",
                start_distance=0, end_distance=2000000):
        super().__init__(path,resolution,chromosome,out_name)
        self.matrix = loadWithNorm(self.path,log = True).values
        self.control_path = control_path
        self.control_matrix = loadWithNorm(self.control_path,log = True).values
        self.start_distance = start_distance
        self.end_distance = end_distance

    def __str__(self):
        return "DirectionalRelativeFreq(matrix_path = '" + str(self.path) +"',\n" + \
                "control_path = '" + str(self.control_path) + "',\n" + \
                "resolution = " + str(self.resolution) + ", chromosome = '" + \
                self.chromosome + "', " + "out_name = '" + self.out_name + \
                "' \n, start_distance = " + str(self.start_distance) + \
                ", end_distance = " + str(self.end_distance) + ")"
    __repr__ = __str__

    def getDRF(self):
        if self.matrix.shape[0] != self.control_matrix.shape[0]:
            print("Error: Input/control matrix have different shape")
            exit(1)

        smooth = 3
        control = ndimage.median_filter(self.control_matrix,smooth)
        logratio = ndimage.median_filter(self.matrix - control, smooth)

        array = np.zeros(self.matrix_shape) * np.NaN
        startDistanceBin = int(self.start_distance / self.resolution)+1
        endDistanceBin = int(self.end_distance / self.resolution)

        for i in range(endDistanceBin, self.matrix_shape - endDistanceBin):
            right = logratio[i+startDistanceBin:i+endDistanceBin+1, i].mean()
            left = logratio[i, i-endDistanceBin:i-startDistanceBin+1].mean()
            if np.isnan(right - left):continue
            array[i] = right - left


        return super().makeDF(array,"DirectionalRelativeFrequency")

    def getCSV(self):
        super().makeCSV(self.getDRF())

class TADScoreChange(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name=""):
        super().__init__(path,resolution,chromosome,out_name)
        self.control_path = control_path

    def getChange(self,mode,parameter):
        if mode == "IS":
            treat = InsulationScore(self.path,self.resolution,self.chromosome,square_size=parameter).getIS().InsulationScore
            control = InsulationScore(self.control_path,self.resolution,self.chromosome,square_size=parameter).getIS().InsulationScore
            title = "InsulationScoreChange"
        elif mode == "CI":
            treat = ContrastIndex(self.path,self.resolution,self.chromosome,CI_size=parameter).getCI().ContrastIndex
            control = ContrastIndex(self.control_path,self.resolution,self.chromosome,CI_size=parameter).getCI().ContrastIndex
            title = "ContrastIndexChange"
        elif mode == "DI":
            treat = DirectionalityIndex(self.path,self.resolution,self.chromosome,DI_distance=parameter).getDI().DirectionalityIndex
            control = DirectionalityIndex(self.control_path,self.resolution,self.chromosome,DI_distance=parameter).getDI().DirectionalityIndex
            title = "DirectionalityIndexChange"
        elif mode == "TADss":
            treat = SeparationScore(self.path,self.resolution,self.chromosome,TADss_size=parameter).getTADss().SeparationScore
            control = SeparationScore(self.control_path,self.resolution,self.chromosome,TADss_size=parameter).getTADss().SeparationScore
            title = "SeparationScoreChange"

        change = np.array(treat - control)
        return super().makeDF(change,title)

    def getCSV(self):
        super().makeCSV(self.getISC())

class deltaDLR(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name="deltaDLR",sizeDLR=3000000):
        super().__init__(path,resolution,chromosome,out_name)
        self.control_path = control_path
        self.sizeDLR = sizeDLR
        self.DLRtreat = DistalToLocalRatio(path,resolution,chromosome,out_name="DLR",
                                            sizeDLR=self.sizeDLR).getDLR()
        self.DLRcontrol= DistalToLocalRatio(control_path,resolution,chromosome,out_name="DLR",
                                            sizeDLR=self.sizeDLR).getDLR()

    def getDeltaDLR(self):
        dDLR = np.array(self.DLRtreat.DistalToLocalRatio - self.DLRcontrol.DistalToLocalRatio)
        return super().makeDF(dDLR,"delta-DistalToLocalRatio")

    def getCSV(self):
        super().makeCSV(self.getDeltaDLR())

class PC1change(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name="",useNA=True,corr_file="",smoothPC=True,logPC=False):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.control_path = control_path
        self.corr_file = corr_file
        self.PC1treat = CompartmentPC1(path,resolution,chromosome,
                                    out_name="PC1").getPC1(signCorr=self.corr_file,smooth = smoothPC, logOE=logPC)
        self.PC1control= CompartmentPC1(control_path,resolution,chromosome,
                                    out_name="PC1").getPC1(signCorr=self.corr_file,smooth = smoothPC, logOE=logPC)

    def getPC1change(self):
        treat = self.PC1treat.CompartmentPC1
        control = self.PC1control.CompartmentPC1
        array = self.blankarray
        for i in range(self.matrix_shape):
            A = treat[i]
            B = control[i]
            if np.isnan(A+B): continue
            if A*B >= 0 :  #同为A/B则视为0
                array[i] = 0
            elif min(abs(treat[i-1:i+2]))<0.01 or min(abs(control[i-1:i+2]))<0.01:
                array[i] = 0
            else:
                array[i] = A - B  #if >0 compartmentBtoA; <0 AtoB

        return super().makeDF(array,"PC1change")

    def getCSV(self):
        super().makeCSV(self.getPC1change())

class CorrelationDifference(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name="",useNA=True,method="pearson"):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.control_path = control_path
        self.method = method
        self.control_matrix = loadDenseMatrix(control_path).values

    def getCorrD(self):
        array = self.blankarray
        for i in range(self.matrix_shape):
            treat = pd.Series(np.log1p(self.matrix[i,:]))
            control = pd.Series(np.log1p(self.control_matrix[i,:]))

            if np.nansum(treat) == 0 or np.nansum(control) == 0:
                continue
            elif np.median(np.nan_to_num(treat)) == 0 or np.median(np.nan_to_num(control)) == 0:
                continue
            else:
                array[i] = treat.corr(control,method=self.method)
        return super().makeDF(array,"CorrelationDifference")

    def getCSV(self):
        super().makeCSV(self.getCorrD())

class intraScoreChange(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name="",useNA=True,IS_size=300000):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.control_path = control_path

        self.treat = intraTADscore(path,resolution,chromosome).getIntraS(IS_size=IS_size,TADpath=control_path).IntraTADscore
        self.control = intraTADscore(control_path,resolution,chromosome).getIntraS(IS_size=IS_size,TADpath=control_path).IntraTADscore

    def getIntraSC(self):
        array = self.blankarray
        for i in range(self.matrix_shape):
            t = np.array(self.treat)[i]
            c = np.array(self.control)[i]
            if np.isnan(t+c) or min(t,c) == 0:
                continue

            array[i] = np.log2(t/c)
        return super().makeDF(array,"IntraTADscoreChange")

    def getCSV(self):
        super().makeCSV(self.getIntraSC())

class interScoreChange(BasePara):
    def __init__(self,path,control_path,resolution,chromosome,out_name="",useNA=True,IS_size=300000):
        super().__init__(path,resolution,chromosome,out_name,useNA)
        self.control_path = control_path

        self.treat = interTADscore(path,resolution,chromosome).getInterS(IS_size=IS_size,TADpath=control_path).InterTADscore
        self.control = interTADscore(control_path,resolution,chromosome).getInterS(IS_size=IS_size,TADpath=control_path).InterTADscore

    def getInterSC(self):
        array = self.blankarray
        for i in range(self.matrix_shape):
            t = np.array(self.treat)[i]
            c = np.array(self.control)[i]
            if np.isnan(t+c) or min(t,c) == 0:
                continue

            array[i] = np.log2(t/c)
        return super().makeDF(array,"InterTADscoreChange")

    def getCSV(self):
        super().makeCSV(self.getInterSC())

class InteractionFrequencyChange:
    def __init__(self,path,control_path,resolution,chromosome,gt=None,datatype="rawhic",normIF=True,out_name="noName"):
        treat = InteractionFrequency(path,resolution,chromosome,gt=gt,datatype=datatype,normIF=normIF).getIF()
        print(control_path)
        control = InteractionFrequency(control_path,resolution,chromosome,gt=gt,datatype=datatype,normIF=normIF).getIF()
        scoreChange= pd.DataFrame({"chr":treat.iloc[:,0],"start":treat.iloc[:,1],
                    "end":treat.iloc[:,2],"InteractionFrequencyChange":treat.iloc[:,3]-control.iloc[:,3]})
        self.score = scoreChange

    def getIFC(self):
        return(self.score)



from multiprocessing import Pool
from .callDirectionalTAD import *
from .MultiTypeScore import *
import os

# calculate each dTAD of chromosome separately
class paralfunc(object):
    def __init__(self,myfun,num_processer):
        self.myfun = myfun
        self.num_processer = num_processer

        #chrlist= list(range(1,23)); chrlist.append("X")
        #chrolist = ["chr"+str(a) for a in chrlist]
        #self.chrolist = chrolist

    def run(self,pathName,pathControl,resolution,maxchr=22):
        chrlist= list(range(1,maxchr+1)); chrlist.append("X")
        chrolist = ["chr"+str(a) for a in chrlist]
        self.chrolist = chrolist

        resultlist=[]
        p = Pool(self.num_processer)
        for r in self.chrolist:
            result = p.apply_async(self.myfun, args=(r,pathName,pathControl,resolution,))
            resultlist.append(result)
        p.close()
        p.join()

        output_left = pd.DataFrame()
        output_right = pd.DataFrame()
        for i in resultlist:
            output_single = i.get().copy()
            output_left = output_left.append(output_single["leftTAD"])
            output_right = output_right.append(output_single["rightTAD"])
        return(output_left,output_right)

def call_dTAD(chrom,pathName,pathControl,resolution):
    filename = pathName+"/observed.KR."+chrom+".matrix.gz"
    controlfilename = pathControl + "/observed.KR."+chrom+".matrix.gz"

    dTAD = DirectionalTAD(filename,controlfilename, \
                      resolution,chr=chrom,startDRF=500000,sizeDRF=2000000,\
                      sizeIS=300000)
    leftTAD,rightTAD,_ = dTAD.extractRegion()
    return({"leftTAD":leftTAD,"rightTAD":rightTAD})

def alldTAD(pathName,pathControl,resolution,outname="twoSample",maxchr=22):
    ltad,rtad = paralfunc(call_dTAD,30).run(pathName,pathControl,resolution,maxchr)
    ltad.to_csv(outname+"_leftTAD.csv",sep="\t",index=False)
    rtad.to_csv(outname+"_rightTAD.csv",sep="\t",index=False)

#alldTAD("/Users/wangjiankang/figureServer/Nov2020/Rad21KD1_HiCmatrix",
#        "/Users/wangjiankang/figureServer/Nov2020/Control1_HiCmatrix",50000,maxchr=23)

#=====================================#
class paralfuncOneSample(object):
    def __init__(self,myfun,num_processer):
        self.myfun = myfun
        self.num_processer = num_processer

    def run(self,pathName,resolution,maxchr=22,type=None,parameter=None,prefix="/observed.KR.",controlpath=None):
        chrlist= list(range(1,maxchr+1)); chrlist.append("X")
        chrolist = ["chr"+str(a) for a in chrlist]
        self.chrolist = chrolist

        resultlist=[]
        p = Pool(self.num_processer)
        for r in self.chrolist:
            result = p.apply_async(self.myfun, args=(r,pathName,resolution,type,parameter,prefix,controlpath))
            resultlist.append(result)
        p.close()
        p.join()

        output_all = pd.DataFrame()
        for i in resultlist:
            output_single = i.get().copy()
            output_all = output_all.append(output_single)
        return(output_all)

def TAD1sample(chrom,pathName,resolution,type,parameter):
    filename = pathName+"/observed.KR."+chrom+".matrix.gz"
    tad = TADcallIS(filename,resolution,chrom,squareSize=300000)
    return(tad)

def runTAD1sample(pathName,resolution,outname="allTAD",maxchr=22):
    tad = paralfuncOneSample(TAD1sample,30).run(pathName,resolution,maxchr)
    tad.to_csv(outname+"_TAD.csv",sep="\t",index=False)

def oneScoreSinglechr(chrom,pathName,resolution,type,parameter,prefix="observed.KR.",controlpath=None):
    filename = pathName+"/"+prefix+chrom+".matrix.gz"
    score = multiScore(filename,resolution,chrom).obtainOneScore(type,parameter)
    return(score)

def oneScoreAllchr(pathName,resolution,type,parameter,outname="OneScore",maxchr=22,prefix="observed.KR.",num=10):
    allscore = paralfuncOneSample(oneScoreSinglechr,num).run(pathName,resolution,maxchr,type,parameter,prefix)
    return(allscore)
    #allscore.to_csv(outname+"_"+type+".csv",sep="\t",index=False)

def multiScoreSinglechr(chrom,pathName,resolution,typelist,parameterlist):
    filename = pathName+"/observed.KR."+chrom+".matrix.gz"
    multiType = multiScore(filename,resolution,chrom).allOneScore(typelist,parameterlist)
    return(multiType)

def multiScoreAllchr(pathName,resolution,typelist,parameterlist,outname="multiScore",maxchr=22):
    allmultiscore = paralfuncOneSample(multiScoreSinglechr,30).run(pathName,resolution,maxchr,typelist,parameterlist)
    return(allmultiscore)

#oneScoreAllchr("/Users/wangjiankang/figureServer/Nov2020/Rad21KD1_HiCmatrix",50000,maxchr=5)


class paralfunJuicer(object):
    def __init__(self,myfun,num_processer):
        self.myfun = myfun
        self.num_processer = num_processer

    def run(self,data,normalize,resolution,gt,outname,maxchr=22):
        chrlist= list(range(1,maxchr+1)); chrlist.append("X")
        chrolist = ["chr"+str(a) for a in chrlist]
        self.chrolist = chrolist

        resultlist=[]
        p = Pool(self.num_processer)
        for r in self.chrolist:
            p.apply_async(self.myfun, args=(r,data,normalize,resolution,gt,outname,))
        p.close()
        p.join()

def oneJuicer(chrom,data,normalize,resolution,gt,outname):
    print("Input data: ", data)
    print("Dumping contact " +chrom+ " matrix from .hic file ......")
    codepath = os.path.dirname(os.path.realpath(__file__))
    makeIntra = codepath+"/extract/makeMatrixIntra.sh"
    juicer = codepath+"/jc/jctool_1.11.04.jar"
    foldername = outname
    os.system("bash "+makeIntra+" "+normalize+" "+"."+" "+data+" "+
            str(resolution)+" "+gt+" "+juicer+" "+chrom+" "+foldername + "> info.txt")
    try: os.system("rm info.txt")
    except: pass
    print("Dump finished, output is in ./"+outname)

def allJuicer(data,normalize,resolution,gt,outname,maxchr=22,num=20):
    paralfunJuicer(oneJuicer,num).run(data,normalize,resolution,gt,outname,maxchr=maxchr)

#========================

class paralfuncTwoSample(object):
    def __init__(self,myfun,num_processer):
        self.myfun = myfun
        self.num_processer = num_processer

    def run(self,pathName,resolution,maxchr=22,type=None,parameter=None,prefix="/observed.KR.",controlpath=None):
        chrlist= list(range(1,maxchr+1)); chrlist.append("X")
        chrolist = ["chr"+str(a) for a in chrlist]
        self.chrolist = chrolist

        resultlist=[]
        p = Pool(self.num_processer)
        for r in self.chrolist:
            result = p.apply_async(self.myfun, args=(r,pathName,resolution,type,parameter,prefix,controlpath))
            resultlist.append(result)
        p.close()
        p.join()

        output_all = pd.DataFrame()
        for i in resultlist:
            output_single = i.get()[0].copy()
            output_all = output_all.append(output_single)
        return(output_all)

def twoScoreSinglechr(chrom,pathName,resolution,type,parameter,prefix="observed.KR.",controlpath=""):
    filename = pathName+"/"+prefix+chrom+".matrix.gz"
    controlfile = controlpath+"/"+prefix+chrom+".matrix.gz"
    score = multiScore(filename,resolution,chrom,control_path=controlfile).obtainTwoScore(type,parameter)
    return(score)

def twoScoreAllchr(pathName,controlpath,resolution,type,parameter,outname="OneScore",maxchr=22,prefix="observed.KR."):
    allscore = paralfuncTwoSample(twoScoreSinglechr,30).run(pathName,resolution,maxchr,type,parameter,prefix,controlpath=controlpath)
    return(allscore)

#allJuicer("/Users/wangjiankang/Documents/localrun/Control_1/inter_30.hic","KR",50000,
#            "./gd/hg38/gd50000.txt","justtest",maxchr=5)



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    readme = fh.read()

setup(
    name='h1d',
    version='0.1.15',
    author='wangjiankng',
    author_email='wangjk321@gmail.com',
    url='https://github.com/wangjk321/HiC1Dmetrics',
    description='HiC1Dmetrics pip version',
    long_description=readme,
    long_description_content_type="text/markdown",
    packages=['h1d'],
    include_package_data=True,
    install_requires=["pandas","numpy","scikit-learn","scipy",
		"matplotlib","seaborn","multiprocess",
		"fithic==2.0.7","statsmodels",'cooler==0.8.2'],
    classifiers=[
        "Programming Language :: Python :: 3",
	"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'h1d=h1d.__main__:CLI',
        ]}
)
